Package: lifecycle
Title: Manage the Life Cycle of your Package Functions
Version: 1.0.4
Authors@R: c(
    person("Lionel", "Henry", , "lionel@posit.co", role = c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Manage the life cycle of your exported functions with shared
    conventions, documentation badges, and user-friendly deprecation
    warnings.
License: MIT + file LICENSE
URL: https://lifecycle.r-lib.org/, https://github.com/r-lib/lifecycle
BugReports: https://github.com/r-lib/lifecycle/issues
Depends: R (>= 3.6)
Imports: cli (>= 3.4.0), glue, rlang (>= 1.1.0)
Suggests: covr, crayon, knitr, lintr, rmarkdown, testthat (>= 3.0.1),
        tibble, tidyverse, tools, vctrs, withr
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate, usethis
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2023-11-06 16:07:36 UTC; lionel
Author: Lionel Henry [aut, cre],
  Hadley Wickham [aut] (<https://orcid.org/0000-0003-4757-117X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2023-11-07 10:10:10 UTC
Built: R 4.4.0; ; 2026-05-11 17:10:59 UTC; unix

Package: lifecycle
Title: Manage the Life Cycle of your Package Functions
Version: 1.0.5
Authors@R: c(
    person("Lionel", "Henry", , "lionel@posit.co", role = c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Manage the life cycle of your exported functions with shared
    conventions, documentation badges, and user-friendly deprecation
    warnings.
License: MIT + file LICENSE
URL: https://lifecycle.r-lib.org/, https://github.com/r-lib/lifecycle
BugReports: https://github.com/r-lib/lifecycle/issues
Depends: R (>= 3.6)
Imports: cli (>= 3.4.0), rlang (>= 1.1.0)
Suggests: covr, knitr, lintr (>= 3.1.0), rmarkdown, testthat (>=
        3.0.1), tibble, tidyverse, tools, vctrs, withr, xml2
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate, usethis
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-07 15:00:49 UTC; lionel
Author: Lionel Henry [aut, cre],
  Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2026-01-08 08:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 10:32:18 UTC; unix

Package: lifecycle
Title: Manage the Life Cycle of your Package Functions
Version: 1.0.5
Authors@R: c(
    person("Lionel", "Henry", , "lionel@posit.co", role = c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Manage the life cycle of your exported functions with shared
    conventions, documentation badges, and user-friendly deprecation
    warnings.
License: MIT + file LICENSE
URL: https://lifecycle.r-lib.org/, https://github.com/r-lib/lifecycle
BugReports: https://github.com/r-lib/lifecycle/issues
Depends: R (>= 3.6)
Imports: cli (>= 3.4.0), rlang (>= 1.1.0)
Suggests: covr, knitr, lintr (>= 3.1.0), rmarkdown, testthat (>=
        3.0.1), tibble, tidyverse, tools, vctrs, withr, xml2
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate, usethis
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-07 15:00:49 UTC; lionel
Author: Lionel Henry [aut, cre],
  Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2026-01-08 08:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,lifecycle_warnings)
export(badge)
export(deprecate_soft)
export(deprecate_stop)
export(deprecate_warn)
export(deprecated)
export(expect_defunct)
export(expect_deprecated)
export(is_present)
export(last_lifecycle_warnings)
export(lint_lifecycle)
export(lint_tidyverse_lifecycle)
export(pkg_lifecycle_statuses)
export(signal_experimental)
export(signal_stage)
export(signal_superseded)
import(rlang)

# lifecycle

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/lifecycle/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/lifecycle/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

lifecycle provides a set of tools and conventions to manage the life cycle of your exported functions.

-   For a general motivation and introduction to the key concepts, watch Hadley's rstudio::global(2021) keynote ["Maintaining the house that the tidyverse built"](https://posit.co/resources/videos/maintaining-the-house-the-tidyverse-built/).

-   Read `vignette("stages")` to learn what it means for a function to be experimental, stable, deprecated, or superseded.

-   Read `vignette("manage")` to learn how to manage lifecycle changes in functions that you use.

-   Read `vignette("communicate")` to learn how to communicate lifecycle changes in the functions you write.

## Installation

``` r
# Install release version from CRAN
install.packages("lifecycle")

# Install development version from GitHub
pak::pak("r-lib/lifecycle")
```


## Code of Conduct

Please note that the lifecycle project is released with a [Contributor Code of Conduct](https://lifecycle.r-lib.org/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

## ----eval = FALSE-------------------------------------------------------------
# df <- tibble::data_frame(x = 1)
# #> Warning message:
# #> `data_frame()` was deprecated in tibble 1.1.0.
# #> Please use `tibble()` instead.
# #> This warning is displayed once every 8 hours.
# #> Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

#' Deprecate functions and arguments
#'
#' @description
#'
#' These functions provide three levels of verbosity for deprecated
#' functions.
#'
#' * `deprecate_soft()` warns only if the deprecated function is
#'   called from the global environment (so the user can change their
#'   script) or from the package currently being tested (so the
#'   package developer can fix the package). Use for soft-deprecated
#'   functions.
#'
#' * `deprecate_warn()` warns unconditionally. Use for deprecated functions.
#'
#' * `deprecate_stop()` fails unconditionally. Use for defunct functions.
#'
#' Warnings are only issued once every 8 hours to avoid overwhelming
#' the user. See the [verbosity option][verbosity] to control this
#' behaviour.
#'
#' Deprecation warnings have class
#' `lifecycle_warning_deprecated`. Deprecation errors have class
#' `lifecycle_error_deprecated`.
#'
#' @param when The package version when function/argument was deprecated.
#' @param what If the deprecated feature is a whole function, the
#'   function name: `"foo()"`. If it's an argument that is being
#'   deprecated, the function call should include the argument:
#'   `"foo(arg = )"`.
#'
#'   You can optionally supply the namespace: `"ns::foo()"`. If not
#'   supplied, it is inferred from the caller environment.
#' @param with An optional replacement for the deprecated feature.
#'   This should be a string of the same form as `what`.
#' @param details The deprecation message is generated from `when`,
#'   `what`, and `with`. You can additionally supply a string
#'   `details` to be appended to the message.
#' @param id The id of the deprecation. A warning is issued only once
#'   for each `id`. Defaults to the generated message, but you should
#'   give a unique ID when the message in `details` is built
#'   programmatically and depends on inputs, or when you'd like to
#'   deprecate multiple functions but warn only once for all of them.
#' @param env The environment in which the deprecated function
#'   was called. A warning is issued if called from the global
#'   environment. If testthat is running, a warning is also called if
#'   the deprecated function was called from the package being tested.
#'
#'   This typically doesn't need to be specified, unless you call
#'   `deprecate_soft()` or `deprecate_warn()` from an internal helper.
#'   In that case, you need to forward the calling environment.
#' @return `NULL`, invisibly.
#'
#' @seealso [lifecycle()]
#'
#' @examples
#' # A deprecated function `foo`:
#' deprecate_warn("1.0.0", "foo()")
#'
#' # A deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg = )")
#'
#' # A partially deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg = 'must be a scalar integer')")
#'
#' # A deprecated function with a function replacement:
#' deprecate_warn("1.0.0", "foo()", "bar()")
#'
#' # A deprecated function with a function replacement from a
#' # different package:
#' deprecate_warn("1.0.0", "foo()", "otherpackage::bar()")
#'
#' # A deprecated function with an argument replacement:
#' deprecate_warn("1.0.0", "foo()", "foo(bar = )")
#'
#' @export
deprecate_soft <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           env = caller_env(2)) {
  verbosity <- lifecycle_verbosity()

  if (verbosity == "quiet") {
    return(invisible(NULL))
  }

  if (verbosity %in% c("warning", "error") || env_inherits_global(env)) {
    deprecate_warn(when, what, with = with, details = details, id = id)
    return(invisible(NULL))
  }

  if (from_testthat(env)) {
    # Warn repeatedly in unit tests
    scoped_options(lifecycle_verbosity = "warning")

    deprecate_warn(when, what, with = with, details = details, id = id)
    return(invisible(NULL))
  }

  msg <- lifecycle_build_message(when, what, with, details, "deprecate_soft")
  signal(msg, "lifecycle_soft_deprecated")
}

# TRUE if we are in unit tests and the package being tested is the
# same as the package that called
from_testthat <- function(env) {
  tested_package <- Sys.getenv("TESTTHAT_PKG")

  # Test for environment names rather than reference/contents because
  # testthat clones the namespace
  nzchar(tested_package) &&
    identical(Sys.getenv("NOT_CRAN"), "true") &&
    env_name(topenv(env)) == env_name(ns_env(tested_package))
}

#' @rdname deprecate_soft
#' @export
deprecate_warn <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           env = caller_env(2)) {
  msg <- lifecycle_build_message(when, what, with, details, "deprecate_warn")
  verbosity <- lifecycle_verbosity()

  id <- id %||% msg
  stopifnot(is_string(id))

  if (verbosity == "quiet") {
    return(invisible(NULL))
  }

  if (verbosity == "default" && !needs_warning(id) && !from_testthat(env)) {
    return(invisible(NULL))
  }

  if (verbosity == "error") {
    deprecate_stop(when, what, with = with, details = details)
  } else {
    if (verbosity == "default") {
      # Prevent warning from being displayed again
      env_poke(deprecation_env, id, Sys.time());

      msg <- paste_line(
        msg,
        silver("This warning is displayed once every 8 hours."),
        silver("Call `lifecycle::last_warnings()` to see where this warning was generated.")
      )
    }

    trace <- trace_back(bottom = caller_env())
    wrn <- new_deprecated_warning(msg, trace)

    # Record muffled warnings if testthat is running because it
    # muffles all warnings but we still want to examine them after a
    # run of `devtools::test()`
    maybe_push_warning <- function() {
      if (Sys.getenv("TESTTHAT_PKG") != "") {
        push_warning(wrn)
      }
    }

    withRestarts(muffleWarning = maybe_push_warning, {
      signalCondition(wrn)
      push_warning(wrn)
      warning(wrn)
    })
  }
}
needs_warning <- function(id) {
  last <- deprecation_env[[id]]
  if (is_null(last)) {
    return(TRUE)
  }

  if (!inherits(last, "POSIXct")) {
    abort("Internal error: Expected `POSIXct` value in `lifecycle::needs_warning()`.")
  }

  # Warn every 8 hours
  (Sys.time() - last) > (8 * 60 * 60)
}

#' @rdname deprecate_soft
#' @export
deprecate_stop <- function(when,
                           what,
                           with = NULL,
                           details = NULL) {
  msg <- lifecycle_build_message(when, what, with, details, "deprecate_stop")

  stop(cnd(
    c("lifecycle_error_deprecated", "defunctError", "error", "condition"),
    old = NULL,
    new = NULL,
    package = NULL,
    message = msg
  ))
}

env_inherits_global <- function(env) {
  # `topenv(emptyenv())` returns the global env. Return `FALSE` in
  # that case to allow passing the empty env when the
  # soft-deprecation should not be promoted to deprecation based on
  # the caller environment.
  if (is_reference(env, empty_env())) {
    return(FALSE)
  }

  is_reference(topenv(env), global_env())
}

lifecycle_validate_message <- function(msg) {
  stopifnot(is_character(msg))
  paste0(msg, collapse = "\n")
}

lifecycle_build_message <- function(when,
                                    what,
                                    with = NULL,
                                    details = chr(),
                                    signaller) {
  details <- details %||% chr()

  stopifnot(
    is_string(when),
    is_string(what),
    is_null(with) || is_string(with),
    is_character(details)
  )

  what <- spec_validate_what(what, "what", signaller)
  fn <- spec_validate_fn(what$call)
  arg <- spec_validate_arg(what$call, signaller)
  reason <- signal_validate_reason(what$call, signaller)

  if (is_null(what$pkg)) {
    env <- topenv(caller_env(2))
    pkg <- signal_validate_pkg(env)
  } else {
    pkg <- what$pkg
  }

  if (is_null(arg)) {
    if (signaller == "deprecate_stop") {
      msg <- glue::glue("`{ fn }()` was deprecated in { pkg } { when } and is now defunct.")
    } else {
      msg <- glue::glue("`{ fn }()` is deprecated as of { pkg } { when }.")
    }
  } else {
    if (signaller == "deprecate_stop" && reason == "is deprecated") {
      msg <- glue::glue("The `{ arg }` argument of `{ fn }()` was deprecated in { pkg } { when } and is now defunct.")
    } else {
      msg <- glue::glue("The `{ arg }` argument of `{ fn }()` { reason } as of { pkg } { when }.")
    }
  }

  if (!is_null(with)) {
    with <- spec_validate_what(with, "with", signaller)
    with_fn <- spec_validate_fn(with$call)
    with_arg <- spec_validate_arg(with$call, signaller)

    with_pkg <- with$pkg %||% pkg
    if (!is_null(with_pkg) && pkg != with_pkg) {
      with_fn <- glue::glue("{ with_pkg }::{ with_fn }")
    }

    if (is_null(with_arg)) {
      please <- glue::glue("Please use `{ with_fn }()` instead.")
    } else if (fn == with_fn) {
      please <- glue::glue("Please use the `{ with_arg }` argument instead.")
    } else {
      please <- glue::glue("Please use the `{ with_arg }` argument of `{ with_fn }()` instead.")
    }

    msg <- paste0(msg, "\n", please)
  }

  paste_line(msg, details)
}

signal_validate_pkg <- function(env) {
  if (is_reference(env, global_env())) {
    # Convenient for experimenting interactively
    return("<NA>")
  }

  if(is_namespace(env)) {
    return(ns_env_name(env))
  }

  abort(glue::glue(
    "
    Internal error: Can't detect the package of the deprecated function.
    Please mention the namespace:

      # Good:
      { signaller }(what = \"namespace::myfunction()\")

      # Bad:
      { signaller }(what = \"myfunction()\")
    "
  ))
}

signal_validate_reason <- function(call, signaller) {
  details <- spec_validate_details(call, signaller)

  if (is_null(details)) {
    "is deprecated"
  } else {
    details
  }
}

test_that("verbosity option is validated", {
  opt <- with_options(lifecycle_verbosity = NULL, lifecycle_verbosity())
  expect_identical(opt, "default")

  expect_warning(
    with_options(lifecycle_verbosity = NA, lifecycle_verbosity()),
    "must be set to one of"
  )
})

test_that("verbosity option has precedence over user env (#113)", {
  mytool <- function() {
    deprecate_soft("1.0.0", "mytool()")
    10 * 10
  }

  rlang::local_options(lifecycle_verbosity = "error")

  expect_error(
    exec(mytool, .env = global_env()),
    class = "defunctError"
  )
})

test_that("signallers require function call syntax for `what`", {
  verify_errors({
    expect_error(deprecate_stop("1.0.0", "foo"), "must have function call syntax")
    expect_error(deprecate_stop("1.0.0", "foo()()"), "must refer to a function name")
    expect_error(deprecate_stop("1.0.0", "foo(arg = , arg = )"), "more than one argument")
    expect_error(deprecate_stop("1.0.0", "foo(arg)"), "in the LHS")
    expect_error(deprecate_stop("1.0.0", "foo(arg = arg)"), "in the LHS")
  })
})

test_that("deprecation messages are constructed for functions", {
  expect_known_output(file = test_path("output", "test-signal-message.txt"), {
    cat_ruler("Inferred package name (here it is base b/c of testthat's eval env)")
    cat_line(lifecycle_build_message("1.0.0", "foo()", signaller = "deprecate_stop"))

    cat_ruler("Overridden package name")
    cat_line(lifecycle_build_message("1.0.0", "mypkg::foo()", signaller = "deprecate_stop"))

    cat_ruler("Replacement function")
    cat_line(lifecycle_build_message("1.0.0", "foo()", "bar()", signaller = "deprecate_stop"))

    cat_ruler("Replacement function with overridden package names (1)")
    cat_line(lifecycle_build_message("1.0.0", "foo::quux()", "foofy()", signaller = "deprecate_stop"))

    cat_ruler("Replacement function with overridden package names (2)")
    cat_line(lifecycle_build_message("1.0.0", "foo::quux()", "bar::foofy()", signaller = "deprecate_stop"))

    cat_ruler("Details")
    details <- glue::glue(
      "

        # Before:
        foo()

        # After:
        bar()
      "
    )
    cat_line(lifecycle_build_message("1.0.0", "foo()", "bar()", details = details, signaller = "deprecate_stop"))
  })
})

test_that("deprecation messages are constructed for arguments", {
  expect_known_output(file = test_path("output", "test-signal-message-args.txt"), {
    cat_ruler("Deprecated argument")
    cat_line(lifecycle_build_message("1.0.0", "foo(quux = )", signaller = "deprecate_stop"))

    cat_ruler("Deprecated argument with function replacement")
    cat_line(lifecycle_build_message("1.0.0", "foo(quux = )", "bar()", signaller = "deprecate_stop"))

    cat_ruler("Deprecated argument with argument replacement (same function)")
    cat_line(lifecycle_build_message("1.0.0", "foo(quux = )", "foo(foofy = )", signaller = "deprecate_stop"))

    cat_ruler("Deprecated argument with argument replacement (different function)")
    cat_line(lifecycle_build_message("1.0.0", "foo(quux = )", "bar(foofy = )", signaller = "deprecate_stop"))

    cat_ruler("Deprecated argument with argument replacement (different function, different package)")
    cat_line(lifecycle_build_message("1.0.0", "aaa::foo(quux = )", "zzz::bar(foofy = )", signaller = "deprecate_stop"))

    cat_ruler("Deprecated argument with reason")
    cat_line(lifecycle_build_message("1.0.0", "aaa::foo(quux = 'can\\'t be a baz')", signaller = "deprecate_stop"))
  })
})

test_that("non-syntactic names are handled gracefully", {
  expect_known_output(file = test_path("output", "test-signal-message-non-syntactic.txt"), {
    cat_ruler("Non-syntactic function name and non-syntactic parameter name")
    cat_line(lifecycle_build_message("1.0.0", "bar::`foo-fy`(`qu-ux` = )", signaller = "deprecate_stop"))
  })
})

test_that("defunct errors inherit from lifecycle subclass", {
  expect_error(deprecate_stop("1.0.0", "foo()"), class = "lifecycle_error_deprecated")
})

test_that("warning conditions are signaled only once if warnings are suppressed", {
  scoped_options(lifecycle_verbosity = "warning")

  x <- 0L
  suppressWarnings(withCallingHandlers(
    warning = function(...) x <<- x + 1L,
    deprecate_warn("1.0.0", "foo()")
  ))

  expect_identical(x, 1L)
})

test_that("signal-deprecated.R produces correct error messages", {
  verify_output(test_path("error", "test-signal-deprecated.txt"), {
    "# signallers require function call syntax for `what`"
    deprecate_stop("1.0.0", "foo")
    deprecate_stop("1.0.0", "foo()()")
    deprecate_stop("1.0.0", "foo(arg = , arg = )")
    deprecate_stop("1.0.0", "foo(arg)")
    deprecate_stop("1.0.0", "foo(arg = arg)")
  })
})

test_that("expect_deprecated() expects lifecycle warnings", {
  fn <- function() deprecate_warn("1.0", "pkg::foo()", id = "expect-deprecated")
  expect_success(expect_deprecated(fn()))
  expect_success(expect_deprecated(fn()))
  expect_failure(expect_deprecated(NULL))
})

test_that("expect_defunct() expects lifecycle errors", {
  fn <- function() deprecate_stop("1.0", "pkg::foo()")
  expect_success(expect_defunct(fn()))
  expect_failure(expect_defunct(NULL))
})

#' Control the verbosity of deprecation signals
#'
#' @description
#'
#' There are 3 levels of verbosity for deprecated functions: silence,
#' warning, and error. Since the lifecycle package avoids disruptive
#' warnings, the default level of verbosity depends on the lifecycle
#' stage of the deprecated function, on the context of the caller
#' (global environment or testthat unit tests cause more warnings),
#' and whether the warning was already issued (see the help for
#' [deprecation functions][deprecate_soft]).
#'
#' You can control the level of verbosity with the global option
#' `lifecycle_verbosity`. It can be set to:
#'
#' * `"quiet"` to suppress all deprecation messages.
#' * `"default"` or `NULL` to warn once every 8 hours.
#' * `"warning"` to warn every time.
#' * `"error"` to error instead of warning.
#'
#' Note that functions calling [deprecate_stop()] invariably throw
#' errors.
#'
#' @examples
#' if (rlang::is_installed("testthat")) {
#'   library(testthat)
#'
#'   mytool <- function() {
#'     deprecate_soft("1.0.0", "mytool()")
#'     10 * 10
#'   }
#'
#'   # Forcing the verbosity level is useful for unit testing. You can
#'   # force errors to test that the function is indeed deprecated:
#'   test_that("mytool is deprecated", {
#'     rlang::local_options(lifecycle_verbosity = "error")
#'     expect_error(mytool(), class = "defunctError")
#'   })
#'
#'   # Or you can enforce silence to safely test that the function
#'   # still works:
#'   test_that("mytool still works", {
#'     rlang::local_options(lifecycle_verbosity = "quiet")
#'     expect_equal(mytool(), 100)
#'   })
#' }
#' @name verbosity
NULL

lifecycle_verbosity <- function() {
  opt <- peek_option("lifecycle_verbosity") %||% "default"

  if (!is_string(opt, c("quiet", "default", "warning", "error"))) {
    options(lifecycle_verbosity = "default")
    warn(glue::glue(
      "
      The `lifecycle_verbosity` option must be set to one of:
      \"quiet\", \"default\", \"warning\", or \"error\".
      Resetting to \"default\".
      "
    ))
  }

  opt
}

test_that("deprecate_soft() warns when called from global env", {
  withr::local_envvar(TESTTHAT_PKG = "testpackage")

  fn <- function(id) {
    deprecate_soft("1.0.0", "foo()", id = id)
  }
  expect_no_warning(fn("called from local env"))

  local_bindings(.env = global_env(), fn = fn)
  env_bind_lazy(
    current_env(),
    do = fn("called from global env"),
    .eval_env = global_env()
  )

  expect_deprecated(do, "foo")
})

test_that("deprecate_soft() warns when called from package being tested", {
  old <- Sys.getenv("NOT_CRAN")
  on.exit(Sys.setenv("NOT_CRAN" = old))

  Sys.setenv("NOT_CRAN" = "true")
  retired <- function() deprecate_soft("1.0.0", "foo()")
  expect_warning(retired(), "was deprecated")
})

test_that("deprecate_soft() warns when option is set", {
  retired <- function(id) deprecate_soft("1.0.0", "foo()", id = id)
  with_options(lifecycle_verbosity = "warning", {
    expect_warning(retired("rlang_test5"), "was deprecated")
  })
})

test_that("deprecate_warn() repeats warnings when option is set", {
  local_options(lifecycle_verbosity = "warning")

  retired1 <- function() deprecate_soft("1.0.0", "foo()", id = "signal repeat")
  retired2 <- function() deprecate_warn("1.0.0", "foo()", id = "warn repeat")

  expect_warning(retired1(), "was deprecated")
  expect_warning(retired2(), "was deprecated")

  expect_warning(retired1(), "was deprecated")
  expect_warning(retired2(), "was deprecated")
})

test_that("can promote lifecycle warnings to errors", {
  local_options(lifecycle_verbosity = "error")
  expect_lifecycle_defunct(deprecate_soft("1.0.0", "foo()"), "was deprecated")
  expect_lifecycle_defunct(deprecate_warn("1.0.0", "foo()"), "was deprecated")
})

test_that("soft-deprecation warnings are issued when called from child of global env as well", {
  fn <- function() deprecate_soft("1.0.0", "foo()", id = "child of global env")
  expect_warning(eval_bare(call2(fn), env(global_env())), "was deprecated")
})

test_that("deprecation warnings are not displayed again", {
  local_options(lifecycle_verbosity = NULL)
  local_interactive()

  wrn <- catch_cnd(
    deprecate_warn("1.0.0", "foo()", id = "once-every-8-hours-note"),
    classes = "warning"
  )
  footer <- cnd_footer(wrn)
  expect_true(is_character(footer) && any(grepl("once every 8 hours", footer)))

  local_options(lifecycle_verbosity = "warning")

  wrn <- catch_cnd(deprecate_warn("1.0.0", "foo()", id = "once-every-8-hours-no-note"))
  expect_false(grepl("once every 8 hours", wrn$message))
})

test_that("the topenv of the empty env is not the global env", {
  local_options(lifecycle_verbosity = NULL)
  expect_silent(deprecate_soft("1.0.0", "foo()", env = empty_env(), id = "topenv of empty env"))
})

test_that("expect_deprecated() matches regexp", {
  expect_deprecated(
    deprecate_warn("1.0", "fn()", details = "foo.["), "foo.[", fixed = TRUE
  )

  fn <- function() {
    deprecate_soft("1.0.0", "fn()")
  }
  expect_deprecated(fn(), "fn")
  expect_deprecated(expect_failure(
    expect_deprecated(fn(), "foo")
  ))
})

# nocov start --- compat-zeallot --- 2020-11-23

# This drop-in file implements a simple version of zeallot::`%<-%`.
# Please find the most recent version in rlang's repository.


`%<-%` <- function(lhs, value) {
  lhs <- substitute(lhs)
  env <- caller_env()

  if (!is_call(lhs, "c")) {
    abort("The left-hand side of `%<-%` must be a call to `c()`.")
  }

  vars <- as.list(lhs[-1])

  if (length(value) != length(vars)) {
    abort("The left- and right-hand sides of `%<-%` must be the same length.")
  }

  for (i in seq_along(vars)) {
    var <- vars[[i]]
    if (!is_symbol(var)) {
      abort(paste0("Element ", i, " of the left-hand side of `%<-%` must be a symbol."))
    }

    env[[as_string(var)]] <- value[[i]]
  }

  invisible(value)
}


# nocov end

#' Deprecate functions and arguments
#'
#' @description
#' These functions provide three levels of verbosity for deprecated
#' functions. Learn how to use them in `vignette("communicate")`.
#'
#' * `deprecate_soft()` warns only if the deprecated function is called
#'   directly, i.e. a user is calling a function they wrote in the global
#'   environment or a developer is calling it in their package. It does not
#'   warn when called indirectly, i.e. the deprecation comes from code that
#'   you don't control.
#'
#' * `deprecate_warn()` warns unconditionally.
#'
#' * `deprecate_stop()` fails unconditionally.
#'
#' Warnings are only issued once every 8 hours to avoid overwhelming
#' the user. Control with [`options(lifecycle_verbosity)`][verbosity].
#'
#' @section Conditions:
#' * Deprecation warnings have class `lifecycle_warning_deprecated`.
#' * Deprecation errors have class `lifecycle_error_deprecated`.
#'
#' @param when A string giving the version when the behaviour was deprecated.
#' @param what A string describing what is deprecated:
#'
#'   * Deprecate a whole function with `"foo()"`.
#'   * Deprecate an argument with `"foo(arg)"`.
#'   * Partially deprecate an argument with
#'     `"foo(arg = 'must be a scalar integer')"`.
#'   * Deprecate anything else with a custom message by wrapping it in `I()`.
#'
#'   You can optionally supply the namespace: `"ns::foo()"`, but this is
#'   usually not needed as it will be inferred from the caller environment.
#'
#' @param with An optional string giving a recommended replacement for the
#'   deprecated behaviour. This takes the same form as `what`.
#' @param details In most cases the deprecation message can be
#'   automatically generated from `with`. When it can't, use `details`
#'   to provide a hand-written message.
#'
#'   `details` can either be a single string or a character vector,
#'   which will be converted to a [bulleted list][cli::cli_bullets].
#'   By default, info bullets are used. Provide a named vectors to
#'   override.
#' @param id The id of the deprecation. A warning is issued only once
#'   for each `id`. Defaults to the generated message, but you should
#'   give a unique ID when the message in `details` is built
#'   programmatically and depends on inputs, or when you'd like to
#'   deprecate multiple functions but warn only once for all of them.
#' @param env,user_env Pair of environments that define where `deprecate_*()`
#'   was called (used to determine the package name) and where the function
#'   called the deprecating function was called (used to determine if
#'   `deprecate_soft()` should message).
#'
#'   These are only needed if you're calling `deprecate_*()` from an internal
#'   helper, in which case you should forward `env = caller_env()` and
#'   `user_env = caller_env(2)`.
#' @return `NULL`, invisibly.
#'
#' @seealso [lifecycle()]
#'
#' @examples
#' # A deprecated function `foo`:
#' deprecate_warn("1.0.0", "foo()")
#'
#' # A deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg)")
#'
#' # A partially deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg = 'must be a scalar integer')")
#'
#' # A deprecated function with a function replacement:
#' deprecate_warn("1.0.0", "foo()", "bar()")
#'
#' # A deprecated function with a function replacement from a
#' # different package:
#' deprecate_warn("1.0.0", "foo()", "otherpackage::bar()")
#'
#' # A deprecated function with custom message:
#' deprecate_warn(
#'   when = "1.0.0",
#'   what = "foo()",
#'   details = "Please use `otherpackage::bar(foo = TRUE)` instead"
#' )
#'
#' # A deprecated function with custom bulleted list:
#' deprecate_warn(
#'   when = "1.0.0",
#'   what = "foo()",
#'   details = c(
#'     x = "This is dangerous",
#'     i = "Did you mean `safe_foo()` instead?"
#'   )
#' )
#' @export
deprecate_soft <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           env = caller_env(),
                           user_env = caller_env(2)) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, signaller = "deprecate_soft")
  signal_stage("deprecated", what)

  verbosity <- lifecycle_verbosity()
  direct <- is_direct(user_env)

  invisible(switch(
    verbosity,
    quiet = NULL,
    warning = ,
    default =
      if (direct) {
        always <- verbosity == "warning"
        trace_env <- caller_env()
        deprecate_warn0(
          msg,
          id,
          always = always,
          direct = TRUE,
          trace_env = trace_env,
          user_env = user_env
        )
      },
    error = deprecate_stop0(msg)
  ))
}

#' @rdname deprecate_soft
#' @param always If `FALSE`, the default, will warn every 8 hours.  If
#'   `TRUE`, will always warn in direct usages. Indirect usages keep
#'   warning every 8 hours to avoid disrupting users who can't fix the
#'   issue. Only use `always = TRUE` after at least one release with
#'   the default.
#' @export
deprecate_warn <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           always = FALSE,
                           env = caller_env(),
                           user_env = caller_env(2)) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, signaller = "deprecate_warn")
  signal_stage("deprecated", what)

  verbosity <- lifecycle_verbosity()

  invisible(switch(
    verbosity,
    quiet = NULL,
    warning = ,
    default = {
      direct <- is_direct(user_env)
      always <- direct && (always || verbosity == "warning")
      trace_env <- caller_env()
      deprecate_warn0(
        msg,
        id,
        always = always,
        direct = direct,
        trace_env = trace_env,
        user_env = user_env
      )
    },
    error = deprecate_stop0(msg),
  ))
}

#' @rdname deprecate_soft
#' @export
deprecate_stop <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           env = caller_env()) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, signaller =  "deprecate_stop")
  signal_stage("deprecated", what)
  deprecate_stop0(msg)
}

# Signals -----------------------------------------------------------------

deprecate_warn0 <- function(msg,
                            id = NULL,
                            always = FALSE,
                            direct = FALSE,
                            call = caller_env(),
                            trace_env = caller_env(),
                            user_env = caller_env(2)) {
  id <- id %||% paste_line(msg)
  if (!always && !needs_warning(id, call = call)) {
    return()
  }

  # Prevent warning from being displayed again
  env_poke(deprecation_env, id, Sys.time())

  footer <- function(...) {
    footer <- NULL

    if (!direct) {
      top <- topenv(user_env)

      if (is_namespace(top)) {
        pkg <- ns_env_name(top)
        url <- pkg_url_bug(pkg)

        likely_line <- cli::format_inline(
          "The deprecated feature was likely used in the {.pkg {pkg}} package."
        )

        if (is_null(url)) {
          report_line <-
            "Please report the issue to the authors."
        } else {
          report_line <- cli::format_inline(
            "Please report the issue at {.url {url}}."
          )
        }

        footer <- c(
          footer,
          "i" = likely_line,
          " " = report_line
        )
      }
    }

    if (is_interactive()) {
      footer <- c(
        footer,
        if (!always) silver("This warning is displayed once every 8 hours."),
        silver("Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.")
      )
    }

    footer
  }

  trace <- trace_back(bottom = trace_env)

  wrn <- new_deprecated_warning(msg, trace, footer = footer)

  # Record muffled warnings if testthat is running because it
  # muffles all warnings but we still want to examine them after a
  # run of `devtools::test()`
  maybe_push_warning <- function() {
    if (Sys.getenv("TESTTHAT_PKG") != "") {
      push_warning(wrn)
    }
  }

  withRestarts(muffleWarning = maybe_push_warning, {
    signalCondition(wrn)
    push_warning(wrn)
    warning(wrn)
  })
}

deprecate_stop0 <- function(msg) {
  cnd_signal(error_cnd(
    c("lifecycle_error_deprecated", "defunctError"),
    old = NULL,
    new = NULL,
    package = NULL,
    message = msg
  ))
}

# Messages ----------------------------------------------------------------

lifecycle_message <- function(when,
                              what,
                              with = NULL,
                              details = NULL,
                              env = caller_env(2),
                              call = caller_env(),
                              signaller = "signal_lifecycle") {
  check_string(when, call = call)

  if (is_null(details)) {
    details <- chr()
  } else {
    check_character(details, call = call)
  }

  what <- spec(what, env, signaller = signaller)
  msg <- lifecycle_message_what(what, when)

  if (!is_null(with)) {
    with <- spec(with, NULL, signaller = signaller)
    msg <- c(msg, "i" = lifecycle_message_with(with, what))
  }

  if (is_null(names(details))) {
    details <- set_names(details, "i")
  }

  c(msg, details)
}

lifecycle_message_what <- function(what, when) {
  glue_what <- function(x) glue::glue_data(what, x)

  if (!inherits(what$fn, "AsIs")) {
    what$fn <- fun_label(what$fn)
  }

  if (is_null(what$arg)) {
    if (what$from == "deprecate_stop") {
      glue_what("{ fn } was deprecated in { pkg } { when } and is now defunct.")
    } else {
      glue_what("{ fn } was deprecated in { pkg } { when }.")
    }
  } else {
    if (what$from == "deprecate_stop" && is_null(what$reason)) {
      glue_what("The `{ arg }` argument of { fn } was deprecated in { pkg } { when } and is now defunct.")
    } else {
      what$reason <- what$reason %||% "is deprecated"
      glue_what("The `{ arg }` argument of { fn } { reason } as of { pkg } { when }.")
    }
  }
}

fun_label <- function(fn) {
  if (grepl("^`", fn)) {
    fn
  } else {
    paste0("`", fn, "()`")
  }
}

lifecycle_message_with <- function(with, what) {
  glue_with <- function(x) glue::glue_data(with, x)

  if (inherits(with$fn, "AsIs")) {
    glue_with("Please use { fn } instead.")
  } else {
    if (!is_null(with$pkg) && what$pkg != with$pkg) {
      with$fn <- glue_with("{ pkg }::{ fn }")
    }

    if (is_null(with$arg)) {
      glue_with("Please use `{ fn }()` instead.")
    } else if (what$fn == with$fn) {
      glue_with("Please use the `{ arg }` argument instead.")
    } else {
      glue_with("Please use the `{ arg }` argument of `{ fn }()` instead.")
    }
  }
}

# Helpers -----------------------------------------------------------------

is_direct <- function(env) {
  env_inherits_global(env) || from_testthat(env)
}

env_inherits_global <- function(env) {
  # `topenv(emptyenv())` returns the global env. Return `FALSE` in
  # that case to allow passing the empty env when the
  # soft-deprecation should not be promoted to deprecation based on
  # the caller environment.
  if (is_reference(env, empty_env())) {
    return(FALSE)
  }

  is_reference(topenv(env), global_env())
}

# TRUE if we are in unit tests and the package being tested is the
# same as the package that called
from_testthat <- function(env) {
  tested_package <- Sys.getenv("TESTTHAT_PKG")
  if (!nzchar(tested_package)) {
    return(FALSE)
  }

  top <- topenv(env)
  if (!is_namespace(top)) {
    return(FALSE)
  }

  # Test for environment names rather than reference/contents because
  # testthat clones the namespace
  identical(ns_env_name(top), tested_package)
}

needs_warning <- function(id, call = caller_env()) {
  check_string(id, call = call)

  last <- deprecation_env[[id]]
  if (is_null(last)) {
    return(TRUE)
  }

  if (!inherits(last, "POSIXct")) {
    abort(
      "Expected `POSIXct` value in `needs_warning()`.",
      .internal = TRUE
    )
  }

  # Warn every 8 hours
  (Sys.time() - last) > (8 * 60 * 60)
}

# lifecycle verbosity -----------------------------------------------------

test_that("default deprecations behave as expected", {
  on.exit(env_unbind(deprecation_env, "test"))
  local_options(lifecycle_verbosity = "default")

  deprecated_feature <- function(...) deprecate_warn("1.0.0", "foo()", with = "bar()", id = "test", ...)
  c(direct, indirect) %<-% new_callers(deprecated_feature)

  expect_snapshot({
    (expect_warning(direct(), class = "lifecycle_warning_deprecated"))
  })
  expect_warning(indirect(), NA)
  expect_warning(indirect(), NA)

  expect_snapshot({
    (expect_defunct(deprecate_stop("1.0.0", "foo()")))
  })
})

test_that("deprecate_warn() only warns repeatedly if always = TRUE", {
  on.exit(env_unbind(deprecation_env, "test"))
  local_options(lifecycle_verbosity = "default")

  deprecated_feature <- function(...) deprecate_warn("1.0.0", "foo()", id = "test", ...)
  c(direct, indirect) %<-% new_callers(deprecated_feature)

  expect_snapshot({
    direct()
    direct()
    indirect()
    indirect()
  })

  expect_snapshot({
    direct(always = TRUE)
    direct(always = TRUE)
    indirect(always = TRUE)
    indirect(always = TRUE)
  })
})

test_that("indirect usage recommends contacting authors", {
  on.exit(env_unbind(deprecation_env, c("test_base", "test_rlang")))
  local_options(lifecycle_verbosity = "default")

  deprecated_feature <- function(..., id) deprecate_warn("1.0.0", "foo()", id = id, ...)
  c(direct, indirect) %<-% new_callers(deprecated_feature)

  # To test for URL
  indirect_rlang <- indirect
  environment(indirect_rlang) <- ns_env("rlang")

  expect_snapshot({
    indirect(id = "test_base")
    indirect_rlang(id = "test_rlang")
  })
})

test_that("quiet suppresses _soft and _warn", {
  local_options(lifecycle_verbosity = "quiet")

  expect_warning(deprecate_soft("1.0.0", "foo()"), NA)
  expect_warning(deprecate_warn("1.0.0", "foo()"), NA)
  expect_defunct(deprecate_stop("1.0.0", "foo()"))
})

test_that("warning always warns in _soft and _warn", {
  local_options(lifecycle_verbosity = "warning")

  expect_deprecated(deprecate_warn("1.0.0", "foo()"))
  expect_defunct(deprecate_stop("1.0.0", "foo()"))
})

test_that("error coverts _soft and _warn to errors", {
  local_options(lifecycle_verbosity = "error")

  expect_defunct(deprecate_soft("1.0.0", "foo()"))
  expect_defunct(deprecate_warn("1.0.0", "foo()"))
  expect_defunct(deprecate_stop("1.0.0", "foo()"))
})

test_that("soft deprecation uses correct calling envs", {
  withr::local_envvar(TESTTHAT_PKG = "testpackage")

  # Simulate package functions available from global environment
  env <- new_environment(parent = ns_env("lifecycle"))
  local(envir = env, {
    softly <- function() {
      deprecate_soft("1.0.0", "softly()")
    }
    softly_softly <- function() {
      softly()
    }
  })
  local_bindings(!!!as.list(env), .env = global_env())

  # Calling package function directly should warn
  cnd <- catch_cnd(evalq(softly(), global_env()), "warning")
  expect_s3_class(cnd, class = "lifecycle_warning_deprecated")
  expect_match(conditionMessage(cnd), "lifecycle")

  # Calling package function indirectly from global env shouldn't
  cnd <- catch_cnd(evalq(softly_softly(), global_env()), "warning")
  expect_equal(cnd, NULL)
})

test_that("warning conditions are signaled only once if warnings are suppressed", {
  local_options(lifecycle_verbosity = "warning")

  deprecated_feature <- function(...) deprecate_warn(...)
  c(direct, indirect) %<-% new_callers(deprecated_feature)

  x <- 0L
  suppressWarnings(withCallingHandlers(
    warning = function(...) x <<- x + 1L, {
      direct("1.0.0", "foo()")
      indirect("1.0.0", "foo()")
    }
  ))

  expect_identical(x, 1L)
})

# messaging ---------------------------------------------------------------

test_that("what deprecation messages are readable", {
  expect_snapshot({
    cat_line(lifecycle_message("1.0.0", "foo()"))
    cat_line(lifecycle_message("1.0.0", "foo()", signaller = "deprecate_stop"))
    cat_line(lifecycle_message("1.0.0", "foo(arg)"))
    cat_line(lifecycle_message("1.0.0", "foo(arg)", signaller = "deprecate_stop"))
    cat_line(lifecycle_message("1.0.0", I("Use of bananas")))
    cat_line(lifecycle_message("1.0.0", I("Use of bananas"), signaller = "deprecate_stop"))
  })
})

test_that("replace deprecation messages are readable", {
  expect_snapshot({
    cat_line(lifecycle_message("1.0.0", "foo()", "package::bar()"))

    cat_line(lifecycle_message("1.0.0", "foo()", "bar()"))
    cat_line(lifecycle_message("1.0.0", "foo(arg1)", "foo(arg2)"))
    cat_line(lifecycle_message("1.0.0", "foo(arg)", "bar(arg)"))

    cat_line(lifecycle_message("1.0.0", I("Use of bananas"), I("apples")))
  })
})

test_that("unusual names are handled gracefully", {
  expect_snapshot({
    cat_line(lifecycle_message("1.0.0", "`foo-fy`(`qu-ux` = )"))
    cat_line(lifecycle_message("1.0.0", "`foo<-`()"))
    cat_line(lifecycle_message("1.0.0", "`+`()"))
  })
})

test_that("details uses an info bullet by default", {
  on.exit(env_unbind(deprecation_env, "test"))
  expect_snapshot({
    deprecate_warn(
      "1.0.0", "foo()",
      details = "Please do that instead.",
      id = "test"
    )
  })

  env_unbind(deprecation_env, "test")
  expect_snapshot({
    deprecate_warn(
      "1.0.0", "foo()",
      details = c("Please do that instead.", "Also know that."),
      id = "test"
    )
  })
})

test_that("can use bullets in details ", {
  on.exit(env_unbind(deprecation_env, "test"))
  expect_snapshot({
    deprecate_warn(
      "1.0.0", "foo()",
      details = c(
        "Unnamed",
        i = "Informative",
        x = "Error"
      ),
      id = "test"
    )
  })
})

test_that("checks input types", {
  expect_snapshot(lifecycle_message(1), error = TRUE)
  expect_snapshot(lifecycle_message("1", details = 1), error = TRUE)
})

# helpers -----------------------------------------------------------------

test_that("env_inherits_global works for simple cases", {
  expect_false(env_inherits_global(empty_env()))

  env <- new_environment(parent = global_env())
  expect_true(env_inherits_global(env))
})

test_that("needs_warning works as expected", {
  on.exit(env_unbind(deprecation_env, "test"))

  expect_snapshot(needs_warning(1), error = TRUE)
  expect_true(needs_warning("test"))

  env_poke(deprecation_env, "test", Sys.time())
  expect_false(needs_warning("test"))

  env_poke(deprecation_env, "test", Sys.time() - 9 * 60 * 60)
  expect_false(needs_warning("test"))

  env_poke(deprecation_env, "test", "x")
  expect_snapshot_error(needs_warning("test"))
})

## ---- eval = FALSE------------------------------------------------------------
#  df <- tibble::data_frame(x = 1)
#  #> Warning message:
#  #> `data_frame()` is deprecated as of tibble 1.1.0.
#  #> Please use `tibble()` instead.
#  #> This warning is displayed once every 8 hours.
#  #> Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

#' Signal other experimental or superseded features
#'
#' @description
#' `r badge("experimental")`
#'
#' `signal_stage()` allows you to signal life cycle stages other than
#' deprecation (for which you should use [deprecate_warn()] and friends). There
#' is no behaviour associated with this signal, it is currently purely a way to
#' express intent at the call site. In the future, we hope to replace this with
#' a standardized call to `base::declare()`.
#'
#' @param stage Life cycle stage, either `"experimental"` or `"superseded"`.
#'
#' @param what String describing what feature the stage applies too, using the
#'   same syntax as [deprecate_warn()].
#'
#' @param with An optional string giving a recommended replacement for a
#'   superseded function.
#'
#' @param env `r badge("deprecated")`
#'
#' @export
#' @examples
#' foofy <- function(x, y, z) {
#'   signal_stage("experimental", "foofy()")
#'   x + y / z
#' }
#' foofy(1, 2, 3)
signal_stage <- function(stage, what, with = NULL, env = deprecated()) {
  # Does nothing
  invisible()
}

#' Deprecated functions for signalling lifecycle stages
#'
#' @description
#' `r badge("deprecated")`
#' @name deprecated-signallers
#' @keywords internal
NULL

#' @rdname deprecated-signallers
#' @export
signal_experimental <- function(when, what, env = caller_env()) {
  deprecate_soft(
    "1.1.0",
    what = "signal_experimental()",
    with = "signal_stage()",
    id = "lifecycle_signal_experimental"
  )
  signal_stage("experimental", what)
}

#' @rdname deprecated-signallers
#' @export
signal_superseded <- function(when, what, env = caller_env()) {
  deprecate_soft(
    "1.1.0",
    what = "signal_superseded()",
    with = "signal_stage()",
    id = "lifecycle_signal_superseded"
  )
  signal_stage("superseded", what)
}

test_that("`signal_stage()` does nothing", {
  expect_null(signal_stage("experimental", "pkg::foo(bar = 'baz')"), NULL)
})

test_that("`signal_experimental()` and `signal_superseded()` are deprecated", {
  expect_snapshot({
    signal_experimental("1.1.0", "foo()")
  })
  expect_snapshot({
    signal_superseded("1.1.0", "foo()")
  })
})

# nocov start --- r-lib/rlang compat-obj-type
#
# Changelog
# =========
#
# 2022-09-16:
# - Unprefixed usage of rlang functions with `rlang::` to
#   avoid onLoad issues when called from rlang (#1482).
#
# 2022-08-11:
# - Prefixed usage of rlang functions with `rlang::`.
#
# 2022-06-22:
# - `friendly_type_of()` is now `obj_type_friendly()`.
# - Added `obj_type_oo()`.
#
# 2021-12-20:
# - Added support for scalar values and empty vectors.
# - Added `stop_input_type()`
#
# 2021-06-30:
# - Added support for missing arguments.
#
# 2021-04-19:
# - Added support for matrices and arrays (#141).
# - Added documentation.
# - Added changelog.


#' Return English-friendly type
#' @param x Any R object.
#' @param value Whether to describe the value of `x`.
#' @param length Whether to mention the length of vectors and lists.
#' @return A string describing the type. Starts with an indefinite
#'   article, e.g. "an integer vector".
#' @noRd
obj_type_friendly <- function(x, value = TRUE, length = FALSE) {
  if (is_missing(x)) {
    return("absent")
  }

  if (is.object(x)) {
    if (inherits(x, "quosure")) {
      type <- "quosure"
    } else {
      type <- paste(class(x), collapse = "/")
    }
    return(sprintf("a <%s> object", type))
  }

  if (!is_vector(x)) {
    return(.rlang_as_friendly_type(typeof(x)))
  }

  n_dim <- length(dim(x))

  if (value && !n_dim) {
    if (is_na(x)) {
      return(switch(
        typeof(x),
        logical = "`NA`",
        integer = "an integer `NA`",
        double = "a numeric `NA`",
        complex = "a complex `NA`",
        character = "a character `NA`",
        .rlang_stop_unexpected_typeof(x)
      ))
    }
    if (length(x) == 1 && !is_list(x)) {
      return(switch(
        typeof(x),
        logical = if (x) "`TRUE`" else "`FALSE`",
        integer = "an integer",
        double = "a number",
        complex = "a complex number",
        character = if (nzchar(x)) "a string" else "`\"\"`",
        raw = "a raw value",
        .rlang_stop_unexpected_typeof(x)
      ))
    }
    if (length(x) == 0) {
      return(switch(
        typeof(x),
        logical = "an empty logical vector",
        integer = "an empty integer vector",
        double = "an empty numeric vector",
        complex = "an empty complex vector",
        character = "an empty character vector",
        raw = "an empty raw vector",
        list = "an empty list",
        .rlang_stop_unexpected_typeof(x)
      ))
    }
  }

  type <- .rlang_as_friendly_vector_type(typeof(x), n_dim)

  if (length && !n_dim) {
    type <- paste0(type, sprintf(" of length %s", length(x)))
  }

  type
}

.rlang_as_friendly_vector_type <- function(type, n_dim) {
  if (type == "list") {
    if (n_dim < 2) {
      return("a list")
    } else if (n_dim == 2) {
      return("a list matrix")
    } else {
      return("a list array")
    }
  }

  type <- switch(
    type,
    logical = "a logical %s",
    integer = "an integer %s",
    numeric = ,
    double = "a double %s",
    complex = "a complex %s",
    character = "a character %s",
    raw = "a raw %s",
    type = paste0("a ", type, " %s")
  )

  if (n_dim < 2) {
    kind <- "vector"
  } else if (n_dim == 2) {
    kind <- "matrix"
  } else {
    kind <- "array"
  }
  sprintf(type, kind)
}

.rlang_as_friendly_type <- function(type) {
  switch(
    type,

    list = "a list",

    NULL = "NULL",
    environment = "an environment",
    externalptr = "a pointer",
    weakref = "a weak reference",
    S4 = "an S4 object",

    name = ,
    symbol = "a symbol",
    language = "a call",
    pairlist = "a pairlist node",
    expression = "an expression vector",

    char = "an internal string",
    promise = "an internal promise",
    ... = "an internal dots object",
    any = "an internal `any` object",
    bytecode = "an internal bytecode object",

    primitive = ,
    builtin = ,
    special = "a primitive function",
    closure = "a function",

    type
  )
}

.rlang_stop_unexpected_typeof <- function(x, call = caller_env()) {
  abort(
    sprintf("Unexpected type <%s>.", typeof(x)),
    call = call
  )
}

#' Return OO type
#' @param x Any R object.
#' @return One of `"bare"` (for non-OO objects), `"S3"`, `"S4"`,
#'   `"R6"`, or `"R7"`.
#' @noRd
obj_type_oo <- function(x) {
  if (!is.object(x)) {
    return("bare")
  }

  class <- inherits(x, c("R6", "R7_object"), which = TRUE)

  if (class[[1]]) {
    "R6"
  } else if (class[[2]]) {
    "R7"
  } else if (isS4(x)) {
    "S4"
  } else {
    "S3"
  }
}

#' @param x The object type which does not conform to `what`. Its
#'   `obj_type_friendly()` is taken and mentioned in the error message.
#' @param what The friendly expected type.
#' @param ... Arguments passed to [abort()].
#' @inheritParams args_error_context
#' @noRd
stop_input_type <- function(x,
                            what,
                            ...,
                            arg = caller_arg(x),
                            call = caller_env()) {
  # From compat-cli.R
  format_arg <- env_get(
    nm = "format_arg",
    last = topenv(),
    default = NULL,
    inherit = TRUE
  )
  if (!is.function(format_arg)) {
    format_arg <- function(x) sprintf("`%s`", x)
  }

  message <- sprintf(
    "%s must be %s, not %s.",
    format_arg(arg),
    what,
    obj_type_friendly(x)
  )

  abort(message, ..., call = call, arg = arg)
}

# nocov end

#' Deprecate functions and arguments
#'
#' @description
#' These functions provide three levels of verbosity for deprecated
#' functions. Learn how to use them in `vignette("communicate")`.
#'
#' * `deprecate_soft()` warns only if the deprecated function is
#'   called from the global environment or from the package currently
#'   being tested.
#'
#' * `deprecate_warn()` warns unconditionally.
#'
#' * `deprecate_stop()` fails unconditionally.
#'
#' Warnings are only issued once every 8 hours to avoid overwhelming
#' the user. Control with [`options(lifecycle_verbosity)`][verbosity].
#'
#' @section Conditions:
#' * Deprecation warnings have class `lifecycle_warning_deprecated`.
#' * Deprecation errors have class `lifecycle_error_deprecated`.
#'
#' @param when A string giving the version when the behaviour was deprecated.
#' @param what A string describing what is deprecated:
#'
#'   * Deprecate a whole function with `"foo()"`.
#'   * Deprecate an argument with `"foo(arg)"`.
#'   * Partially deprecate an argument with
#'     `"foo(arg = 'must be a scalar integer')"`.
#'   * Deprecate anything else with a custom message by wrapping it in `I()`.
#'
#'   You can optionally supply the namespace: `"ns::foo()"`, but this is
#'   usually not needed as it will be inferred from the caller environment.
#'
#' @param with An optional string giving a recommended replacement for the
#'   deprecated behaviour. This takes the same form as `what`.
#' @param details In most cases the deprecation message can be automatically
#'   generated from `with`. When it can't, use `details` to provide a
#'   hand-written message. `details` can either be a single string or a
#'   character vector, which will be converted to a bulleted list.
#' @param id The id of the deprecation. A warning is issued only once
#'   for each `id`. Defaults to the generated message, but you should
#'   give a unique ID when the message in `details` is built
#'   programmatically and depends on inputs, or when you'd like to
#'   deprecate multiple functions but warn only once for all of them.
#' @param env,user_env Pair of environments that define where `deprecate_*()`
#'   was called (used to determine the package name) and where the function
#'   called the deprecating function was called (used to determine if
#'   `deprecate_soft()` should message).
#'
#'   These are only needed if you're calling `deprecate_*()` from an internal
#'   helper, in which case you should forward `env = caller_env()` and
#'   `user_env = caller_env(2)`.
#' @return `NULL`, invisibly.
#'
#' @seealso [lifecycle()]
#'
#' @examples
#' # A deprecated function `foo`:
#' deprecate_warn("1.0.0", "foo()")
#'
#' # A deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg)")
#'
#' # A partially deprecated argument `arg`:
#' deprecate_warn("1.0.0", "foo(arg = 'must be a scalar integer')")
#'
#' # A deprecated function with a function replacement:
#' deprecate_warn("1.0.0", "foo()", "bar()")
#'
#' # A deprecated function with a function replacement from a
#' # different package:
#' deprecate_warn("1.0.0", "foo()", "otherpackage::bar()")
#'
#' # A deprecated function with custom message:
#' deprecate_warn(
#'   when = "1.0.0",
#'   what = "foo()",
#'   details = "Please use `otherpackage::bar(foo = TRUE)` instead"
#' )
#'
#' # A deprecated function with custom bulleted list:
#' deprecate_warn(
#'   when = "1.0.0",
#'   what = "foo()",
#'   details = c(
#'     x = "This is dangerous",
#'     i = "Did you mean `safe_foo()` instead?"
#'   )
#' )
#' @export
deprecate_soft <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           env = caller_env(),
                           user_env = caller_env(2)) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, "deprecate_soft")
  signal_stage("deprecated", what)

  verbosity <- lifecycle_verbosity()
  if (verbosity == "quiet") {
    NULL
  } else if (verbosity %in% "warning" ||
             (is_string(verbosity, "default") && env_inherits_global(user_env))) {
    trace <- trace_back(bottom = caller_env())
    deprecate_warn0(msg, trace, always = TRUE)
  } else if (verbosity == "error") {
    deprecate_stop0(msg)
  } else {
    deprecate_soft0(msg)
  }

  invisible(NULL)
}

#' @rdname deprecate_soft
#' @param always If `FALSE`, the default, will warn every 8 hours.
#'   If `TRUE`, will always warn. Only use `always = TRUE` after at least
#'   one release with the default.
#' @export
deprecate_warn <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           id = NULL,
                           always = FALSE,
                           env = caller_env()) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, "deprecate_warn")
  signal_stage("deprecated", what)

  verbosity <- lifecycle_verbosity()
  if (verbosity == "quiet") {
    NULL
  } else if (verbosity == "warning") {
    trace <- trace_back(bottom = caller_env())
    deprecate_warn0(msg, trace, always = TRUE)
  } else if (verbosity == "error") {
    deprecate_stop0(msg)
  } else {
    id <- id %||% msg

    if (always || needs_warning(id)) {
      # Prevent warning from being displayed again
      env_poke(deprecation_env, id, Sys.time())

      trace <- trace_back(bottom = caller_env())
      deprecate_warn0(msg, trace, always = always)
    }
  }

  invisible(NULL)
}

#' @rdname deprecate_soft
#' @export
deprecate_stop <- function(when,
                           what,
                           with = NULL,
                           details = NULL,
                           env = caller_env()) {
  msg <- NULL # trick R CMD check
  msg %<~% lifecycle_message(when, what, with, details, env, "deprecate_stop")
  signal_stage("deprecated", what)
  deprecate_stop0(msg)
}

# Signals -----------------------------------------------------------------

deprecate_soft0 <- function(msg) {
  signal(msg, "lifecycle_soft_deprecated")
}

deprecate_warn0 <- function(msg, trace = NULL, always = FALSE) {
  footer <- function(...) {
    if (is_interactive()) {
      c(
        if (!always) silver("This warning is displayed once every 8 hours."),
        silver("Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.")
      )
    }
  }
  wrn <- new_deprecated_warning(msg, trace, footer = footer)

  # Record muffled warnings if testthat is running because it
  # muffles all warnings but we still want to examine them after a
  # run of `devtools::test()`
  maybe_push_warning <- function() {
    if (Sys.getenv("TESTTHAT_PKG") != "") {
      push_warning(wrn)
    }
  }

  withRestarts(muffleWarning = maybe_push_warning, {
    signalCondition(wrn)
    push_warning(wrn)
    warning(wrn)
  })
}

deprecate_stop0 <- function(msg) {
  stop(cnd(
    c("lifecycle_error_deprecated", "defunctError", "error", "condition"),
    old = NULL,
    new = NULL,
    package = NULL,
    message = msg
  ))
}

# Messages ----------------------------------------------------------------

lifecycle_message <- function(when,
                              what,
                              with = NULL,
                              details = NULL,
                              env = caller_env(2),
                              signaller = "signal_lifecycle") {
  if (!is_string(when)) {
    lifecycle_abort("`when` must be a string")
  }

  details <- details %||% chr()
  if (!is.character(details)) {
    lifecycle_abort("`details` must be a character vector")
  }
  if (length(details) > 1) {
    details <- format_error_bullets(details)
  }

  what <- spec(what, env, signaller)
  msg <- lifecycle_message_what(what, when)

  if (!is_null(with)) {
    with <- spec(with, NULL, signaller)
    msg <- paste0(msg, "\n", lifecycle_message_with(with, what))
  }

  paste_line(msg, details)
}

lifecycle_message_what <- function(what, when) {
  glue_what <- function(x) glue::glue_data(what, x)

  if (!inherits(what$fn, "AsIs")) {
    what$fn <- fun_label(what$fn)
  }

  if (is_null(what$arg)) {
    if (what$from == "deprecate_stop") {
      glue_what("{ fn } was deprecated in { pkg } { when } and is now defunct.")
    } else {
      glue_what("{ fn } was deprecated in { pkg } { when }.")
    }
  } else {
    if (what$from == "deprecate_stop" && is_null(what$reason)) {
      glue_what("The `{ arg }` argument of { fn } was deprecated in { pkg } { when } and is now defunct.")
    } else {
      what$reason <- what$reason %||% "is deprecated"
      glue_what("The `{ arg }` argument of { fn } { reason } as of { pkg } { when }.")
    }
  }
}

fun_label <- function(fn) {
  if (grepl("^`", fn)) {
    fn
  } else {
    paste0("`", fn, "()`")
  }
}

lifecycle_message_with <- function(with, what) {
  glue_with <- function(x) glue::glue_data(with, x)

  if (inherits(with$fn, "AsIs")) {
    glue_with("Please use { fn } instead.")
  } else {
    if (!is_null(with$pkg) && what$pkg != with$pkg) {
      with$fn <- glue_with("{ pkg }::{ fn }")
    }

    if (is_null(with$arg)) {
      glue_with("Please use `{ fn }()` instead.")
    } else if (what$fn == with$fn) {
      glue_with("Please use the `{ arg }` argument instead.")
    } else {
      glue_with("Please use the `{ arg }` argument of `{ fn }()` instead.")
    }
  }
}

# Helpers -----------------------------------------------------------------

env_inherits_global <- function(env) {
  # `topenv(emptyenv())` returns the global env. Return `FALSE` in
  # that case to allow passing the empty env when the
  # soft-deprecation should not be promoted to deprecation based on
  # the caller environment.
  if (is_reference(env, empty_env())) {
    return(FALSE)
  }

  is_reference(topenv(env), global_env())
}

needs_warning <- function(id) {
  if (!is_string(id)) {
    lifecycle_abort("`id` must be a string")
  }

  last <- deprecation_env[[id]]
  if (is_null(last)) {
    return(TRUE)
  }

  if (!inherits(last, "POSIXct")) {
    lifecycle_abort("Expected `POSIXct` value in `needs_warning()`.")
  }

  # Warn every 8 hours
  (Sys.time() - last) > (8 * 60 * 60)
}

upcase1 <- function(x) {
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

cat_line <- function(...) {
  cat(paste0(..., "\n", collapse = ""))
}
paste_line <- function(...) {
  paste(chr(...), collapse = "\n")
}

# nocov start
has_crayon <- function() {
  is_installed("crayon") && crayon::has_color()
}
red       <- function(x) if (has_crayon()) crayon::red(x)       else x
blue      <- function(x) if (has_crayon()) crayon::blue(x)      else x
green     <- function(x) if (has_crayon()) crayon::green(x)     else x
yellow    <- function(x) if (has_crayon()) crayon::yellow(x)    else x
magenta   <- function(x) if (has_crayon()) crayon::magenta(x)   else x
cyan      <- function(x) if (has_crayon()) crayon::cyan(x)      else x
blurred   <- function(x) if (has_crayon()) crayon::blurred(x)   else x
silver    <- function(x) if (has_crayon()) crayon::silver(x)    else x
bold      <- function(x) if (has_crayon()) crayon::bold(x)      else x
italic    <- function(x) if (has_crayon()) crayon::italic(x)    else x
underline <- function(x) if (has_crayon()) crayon::underline(x) else x
# nocov end

pkg_url_bug <- function(pkg) {
  # First check that package is installed, e.g. in case of
  # runtime-only namespace created by pkgload
  if (nzchar(system.file(package = pkg))) {
    url <- utils::packageDescription(pkg)$BugReports

    # `url` can be NULL if not part of the description
    if (is_string(url) && grepl("^https://", url)) {
      return(url)
    }
  }

  NULL
}

# nocov start --- r-lib/rlang compat-types-check
#
# Dependencies
# ============
#
# - compat-obj-type.R
#
# Changelog
# =========
#
# 2022-09-16:
# - Unprefixed usage of rlang functions with `rlang::` to
#   avoid onLoad issues when called from rlang (#1482).
#
# 2022-08-11:
# - Added changelog.

# Scalars -----------------------------------------------------------------

check_bool <- function(x,
                       ...,
                       what = "`TRUE` or `FALSE`",
                       arg = caller_arg(x),
                       call = caller_env()) {
  if (!is_bool(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_string <- function(x,
                         ...,
                         what = "a single string",
                         arg = caller_arg(x),
                         call = caller_env()) {
  if (!is_string(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_number <- function(x,
                         ...,
                         what = "a round number",
                         arg = caller_arg(x),
                         call = caller_env()) {
  if (!is_number(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

is_number <- function(x) {
  is_integerish(x, n = 1, finite = TRUE)
}

check_symbol <- function(x,
                         ...,
                         what = "a symbol",
                         arg = caller_arg(x),
                         call = caller_env()) {
  if (!is_symbol(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_arg <- function(x,
                      ...,
                      what = "an argument name",
                      arg = caller_arg(x),
                      call = caller_env()) {
  check_symbol(x, ..., what = what, arg = arg, call = call)
}

check_call <- function(x,
                       ...,
                       what = "a defused call",
                       arg = caller_arg(x),
                       call = caller_env()) {
  if (!is_call(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_environment <- function(x,
                              ...,
                              what = "an environment",
                              arg = caller_arg(x),
                              call = caller_env()) {
  if (!is_environment(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_function <- function(x,
                           ...,
                           what = "a function",
                           arg = caller_arg(x),
                           call = caller_env()) {
  if (!is_function(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_closure <- function(x,
                           ...,
                           what = "an R function",
                           arg = caller_arg(x),
                           call = caller_env()) {
  if (!is_closure(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

check_formula <- function(x,
                          ...,
                          what = "a formula",
                          arg = caller_arg(x),
                          call = caller_env()) {
  if (!is_formula(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}


# Vectors -----------------------------------------------------------------

# TODO: Restrict missing and special values

check_character <- function(x,
                            ...,
                            what = "a character vector",
                            arg = caller_arg(x),
                            call = caller_env()) {
  if (!is_character(x)) {
    stop_input_type(x, what, ..., arg = arg, call = call)
  }
}

# nocov end

on_load <- function(expr, env = parent.frame(), ns = topenv(env)) {
  expr <- substitute(expr)
  force(env)

  callback <- function() eval_bare(expr, env)
  ns$.__rlang_hook__. <- c(ns$.__rlang_hook__., list(callback))
}

run_on_load <- function(env = parent.frame()) {
  ns <- topenv(env)

  hook <- ns$.__rlang_hook__.
  env_unbind(ns, ".__rlang_hook__.")

  # FIXME: Transform to `while` loop to allow hooking into on-load
  # from an on-load hook?
  for (callback in hook) {
    callback()
  }

  ns$.__rlang_hook__. <- NULL
}


replace_from <- function(pkg, what, to = topenv(caller_env())) {
  if (what %in% getNamespaceExports(pkg)) {
    env <- ns_env(pkg)
  } else {
    env <- to
  }
  env_get(env, what, inherit = TRUE)
}

# Retrieve the lifecycle status defined in each Rd file
db_lifecycle <- function(db) {
  lifecycle_patterns <- paste0(
    "(?:",
    paste(
      collapse = "|",
      c(
        "lifecycle::badge\\([\\\\]\"",
        "rlang:::lifecycle\\([\\\\]\"",
        "list\\(\"lifecycle-",
        "https://www.tidyverse.org/lifecycle/#"
      )
    ),
    ")([\\w-]+)"
  )

  desc <- lapply(db, asNamespace("tools")$.Rd_get_metadata, "description")
  lapply(desc, function(x) {
    res <- regexpr(lifecycle_patterns, x, perl = TRUE)
    starts <- attr(res, "capture.start")
    ends <- starts + attr(res, "capture.length") - 1
    substring(x, starts, ends)
  })
}

# Retrieve the functions listed in the usage for each Rd file in the database
db_function <- function(db) {
  usage <- lapply(db, asNamespace("tools")$.Rd_get_section, "usage")
  lapply(usage, get_usage_function_names)
}

#' @param package The name of an installed package.
#' @param which The lifecycle statuses to retrieve.
#'   Include `NA` if you want to include functions without a specified lifecycle
#'   status in the results.
#' @export
#' @rdname lifecycle_linter
pkg_lifecycle_statuses <- function(
  package,
  which = c(
    "superseded",
    "deprecated",
    "questioning",
    "defunct",
    "experimental",
    "soft-deprecated",
    "retired"
  )
) {
  check_installed("vctrs")
  which <- match.arg(which, several.ok = TRUE)
  stopifnot(is_string(package))

  db <- tools::Rd_db(package)
  lc <- db_lifecycle(db)
  funs <- db_function(db)

  res <- mapply(
    function(lc, f) {
      data.frame(
        fun = f,
        lifecycle = rep(lc, length(f)),
        stringsAsFactors = FALSE
      )
    },
    lc,
    funs,
    SIMPLIFY = FALSE
  )

  res <- vctrs::vec_rbind(!!!res)

  # Filter funs without a lifecycle
  if (!NA %in% which) {
    res <- res[!is.na(res$lifecycle), ]
  }

  # Filter funs without a function name
  res <- res[nzchar(res$fun), ]

  # Filter method definitions
  res <- res[grep("[\\\\]method\\{", res$fun, invert = TRUE), ]

  # filter lifecycles not in which
  res <- res[res$lifecycle %in% which, ]

  if (nrow(res) == 0) {
    return(data.frame(
      package = character(),
      fun = character(),
      lifecycle = character()
    ))
  }

  res$package <- package

  res[c("package", "fun", "lifecycle")]
}

get_usage_function_names <- function(x) {
  if (!length(x)) {
    character(1)
  } else {
    res <- asNamespace("tools")$.parse_usage_as_much_as_possible(x)
    vapply(
      res,
      function(x) {
        if (is.call(x)) as.character(x[[1]]) else character(1)
      },
      character(1)
    )
  }
}

#' @rdname lifecycle_linter
#' @param path The directory path to the files you want to search.
#' @param pattern Any files matching this pattern will be searched. The default
#'   searches any files ending in `.R` or `.Rmd`.
#' @export
lint_lifecycle <- function(
  packages,
  path = ".",
  pattern = "(?i)[.](r|rmd|qmd|rnw|rhtml|rrst|rtex|rtxt)$",
  which = c(
    "superseded",
    "deprecated",
    "questioning",
    "defunct",
    "experimental",
    "soft-deprecated",
    "retired"
  ),
  symbol_is_undesirable = FALSE
) {
  which <- match.arg(which, several.ok = TRUE)

  check_installed(c("lintr", "vctrs", "xml2"))

  lintr::lint_dir(
    path = path,
    pattern = pattern,
    linters = lifecycle_linter(
      packages = packages,
      which = which,
      symbol_is_undesirable = symbol_is_undesirable
    )
  )
}

#' @rdname lifecycle_linter
#' @export
lint_tidyverse_lifecycle <- function(
  path = ".",
  pattern = "(?i)[.](r|rmd|qmd|rnw|rhtml|rrst|rtex|rtxt)$",
  which = c(
    "superseded",
    "deprecated",
    "questioning",
    "defunct",
    "experimental",
    "soft-deprecated",
    "retired"
  ),
  symbol_is_undesirable = FALSE
) {
  which <- match.arg(which, several.ok = TRUE)

  check_installed(c("lintr", "vctrs", "xml2", "tidyverse"))

  lint_lifecycle(
    packages = tidyverse::tidyverse_packages(),
    pattern = pattern,
    path = path,
    which = which,
    symbol_is_undesirable = symbol_is_undesirable
  )
}

#' Lint usages of functions that have a non-stable life cycle.
#'
#' - `lifecycle_linter()` creates a linter for lifecycle annotations which can be
#'   included in a `.lintr` configuration if `lintr` is used directly.
#' - `lint_lifecycle()` dynamically queries the package documentation for packages
#'   in `packages` for lifecycle annotations and then searches the directory in
#'   `path` for usages of those functions.
#' - `lint_tidyverse_lifecycle()` is a convenience function to call `lint_lifecycle()`
#'   for all the packages in the tidyverse.
#' - `pkg_lifecycle_statuses()` returns a data frame of functions with lifecycle
#'   annotations for an installed package.
#'
#' @param packages One or more installed packages to query for lifecycle statuses.
#' @param which Vector of lifecycle statuses to lint.
#' @param symbol_is_undesirable Also lint symbol usages, e.g. `lapply(x, is_na)`?
#'
#' @export
#' @examples
#' lintr::lint(
#'   text = "is_na(x)",
#'   linters = lifecycle_linter(packages = "rlang")
#' )
#' lintr::lint(
#'   text = "lapply(x, is_na)",
#'   linters = lifecycle_linter(packages = "rlang",
#'   symbol_is_undesirable = TRUE)
#' )
lifecycle_linter <- function(
  packages = tidyverse::tidyverse_packages(),
  which = c(
    "superseded",
    "deprecated",
    "questioning",
    "defunct",
    "experimental",
    "soft-deprecated",
    "retired"
  ),
  symbol_is_undesirable = FALSE
) {
  check_installed(c("lintr", "vctrs", "xml2"))

  life_cycles <- vctrs::vec_rbind(
    !!!lapply(packages, pkg_lifecycle_statuses, which = which)
  )
  bad_usages <- sprintf(
    "`%s::%s` is %s",
    life_cycles$package,
    life_cycles$fun,
    life_cycles$lifecycle
  )
  names(bad_usages) <- life_cycles$fun

  if (symbol_is_undesirable) {
    xpath <- sprintf(
      "//SYMBOL_FUNCTION_CALL[%1$s] | //SYMBOL[%1$s]",
      paste0("text() = '", names(bad_usages), "'", collapse = " or ")
    )
  } else {
    xpath <- sprintf(
      "//SYMBOL_FUNCTION_CALL[%s]",
      paste0("text() = '", names(bad_usages), "'", collapse = " or ")
    )
  }

  lintr::Linter(function(source_expression) {
    if (!lintr::is_lint_level(source_expression, "expression")) {
      return(list())
    }

    matched_nodes <- xml2::xml_find_all(
      source_expression$xml_parsed_content,
      xpath
    )
    fun_names <- lintr::get_r_string(matched_nodes)

    lintr::xml_nodes_to_lints(
      matched_nodes,
      source_expression = source_expression,
      lint_message = unname(bad_usages[fun_names])
    )
  })
}

#' Display last deprecation warnings
#'
#' @description
#'
#' `last_lifecycle_warnings()` returns a list of all warnings that
#' occurred during the last top-level R command, along with a
#' backtrace.
#'
#' Use `print(last_lifecycle_warnings(), simplify = level)` to control
#' the verbosity of the backtrace. The `simplify` argument supports
#' one of `"branch"` (the default), `"collapse"`, and `"none"` (in
#' increasing order of verbosity).
#'
#' @examples
#' # These examples are not run because `last_lifecycle_warnings()` does not
#' # work well within knitr and pkgdown
#' \dontrun{
#'
#' f <- function() invisible(g())
#' g <- function() list(h(), i())
#' h <- function() deprecate_warn("1.0.0", "this()")
#' i <- function() deprecate_warn("1.0.0", "that()")
#' f()
#'
#' # Print all the warnings that occurred during the last command:
#' last_lifecycle_warnings()
#'
#'
#' # By default, the backtraces are printed in their simplified form.
#' # Use `simplify` to control the verbosity:
#' print(last_lifecycle_warnings(), simplify = "none")
#' }
#' @export
last_lifecycle_warnings <- function() {
  structure(
    warnings_env$warnings,
    class = c("lifecycle_warnings", "list")
  )
}

new_deprecated_warning <- function(msg, trace, ..., footer = NULL) {
  warning_cnd(
    "lifecycle_warning_deprecated",
    message = msg,
    trace = trace,
    footer = footer,
    internal = list(...)
  )
}

#' @export
print.lifecycle_warnings <- function(x, ...) {
  local_interactive(FALSE)
  print(unclass(x))
}

warnings_env <- env(empty_env())

init_warnings <- function() {
  warnings_env$last_top_frame <- NULL
  warnings_env$warnings <- list()
}
init_warnings()

push_warning <- function(wrn) {
  current <- obj_address(sys.frame(1))

  if (identical(warnings_env$last_top_frame, current)) {
    warnings_env$warnings <- c(warnings_env$warnings, list(wrn))
  } else {
    warnings_env$last_top_frame <- current
    warnings_env$warnings <- list(wrn)
  }
}


# Contains unique IDs of deprecated features so we don't warn multiple times
deprecation_env <- env(empty_env())

test_that("deprecation warning is displayed with backtrace", {
  skip_on_cran()
  skip_on_os("windows")

  init_warnings()

  local_options(
    rlang_trace_top_env = current_env(),
    rlang_trace_format_srcrefs = FALSE,
    crayon.enabled = FALSE
  )

  f <- function() g()
  g <- function() h()
  h <- function() deprecate_warn("1.0.0", "trace()")

  expect_deprecated(f())

  expect_snapshot({
    last_lifecycle_warnings()
  })
})

#' @keywords internal
#' @import rlang
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

.onLoad <- function(lib, pkg) {
  run_on_load()
}

on_load(
  local_use_cli()
)

#' Embed a lifecycle badge in documentation
#'
#' @description
#'
#' To include lifecycle badges in your documentation:
#'
#' 1. Call `usethis::use_lifecycle()` to copy the badge images into the
#'    `man/` folder of your package.
#'
#' 2. Call `lifecycle::badge()` inside R backticks to insert a
#'    lifecycle badge:
#'
#'     ```
#'     #' `r lifecycle::badge("experimental")`
#'     #' `r lifecycle::badge("deprecated")`
#'     #' `r lifecycle::badge("superseded")`
#'     ```
#'
#'    If the deprecated feature is a function, a good place for this
#'    badge is at the top of the topic description. If it is an argument,
#'    you can put the badge in the argument description.
#'
#' The badge is displayed as an image in the HTML version of the
#' documentation and as text otherwise.
#'
#' `lifecycle::badge()` is run by roxygen at build time so you don't need
#' to add lifecycle to `Imports:` just to use the badges. However, it's still
#' good practice to add to `Suggests:` so that it will be available to
#' package developers.
#'
#' @section Badges:
#' * `r lifecycle::badge("experimental")` `lifecycle::badge("experimental")`
#' * `r lifecycle::badge("stable")` `lifecycle::badge("stable")`
#' * `r lifecycle::badge("superseded")` `lifecycle::badge("superseded")`
#' * `r lifecycle::badge("deprecated")` `lifecycle::badge("deprecated")`
#'
#' The meaning of these stages is described in
#' `vignette("stages")`.
#'
#' @param stage A lifecycle stage as a string. Must be one of
#'   `"experimental"`, `"stable"`, `"superseded"`, or `"deprecated"`.
#' @return An `Rd` expression describing the lifecycle stage.
#'
#' @export
badge <- function(stage) {
  old <- c("maturing", "questioning", "soft-deprecated", "defunct", "retired")
  if (!stage %in% old) {
    stage <- arg_match0(
      stage,
      c("experimental", "stable", "superseded", "deprecated")
    )
  }

  url <- paste0("https://lifecycle.r-lib.org/articles/stages.html#", stage)
  html <- sprintf(
    "\\href{%s}{\\figure{%s}{options: alt='[%s]'}}",
    url,
    file.path(sprintf("lifecycle-%s.svg", stage)),
    upcase1(stage)
  )
  text <- sprintf("\\strong{[%s]}", upcase1(stage))

  sprintf("\\ifelse{html}{%s}{%s}", html, text)
}

#' Mark an argument as deprecated
#'
#' Signal deprecated argument by using self-documenting sentinel
#' `deprecated()` as default argument. Test whether the caller has
#' supplied the argument with `is_present()`.
#'
#' @section Magical defaults:
#'
#' We recommend importing `lifecycle::deprecated()` in your namespace
#' and use it without the namespace qualifier.
#'
#' In general, we advise against such magical defaults, i.e. defaults
#' that cannot be evaluated by the user. In the case of
#' `deprecated()`, the trade-off is worth it because the meaning of
#' this default is obvious and there is no reason for the user to call
#' `deprecated()` themselves.
#'
#' @examples
#' foobar_adder <- function(foo, bar, baz = deprecated()) {
#'   # Check if user has supplied `baz` instead of `bar`
#'   if (lifecycle::is_present(baz)) {
#'
#'     # Signal the deprecation to the user
#'     deprecate_warn("1.0.0", "foo::bar_adder(baz = )", "foo::bar_adder(bar = )")
#'
#'     # Deal with the deprecated argument for compatibility
#'     bar <- baz
#'   }
#'
#'   foo + bar
#' }
#'
#' foobar_adder(1, 2)
#' foobar_adder(1, baz = 2)
#' @export
deprecated <- function() {
  missing_arg()
}
#' @rdname deprecated
#' @param arg A `deprecated()` function argument.
#' @export
is_present <- function(arg) {
  !is_missing(maybe_missing(arg))
}

test_that("deprecated() returns the missing argument", {
  fn <- function(foo = deprecated()) is_present(foo)
  expect_false(fn())
  expect_true(fn(1))

  fn <- function(foo) is_present(foo)
  expect_false(fn())
  expect_true(fn(1))
})

spec <- function(spec,
                 env = caller_env(),
                 signaller = "signal_lifecycle",
                 error_call = caller_env()) {
  ctxt <- list(
    signaller = signaller,
    call = error_call
  )

  if (inherits(spec, "AsIs")) {
    list(
      fn = spec,
      arg = NULL,
      pkg = spec_pkg(NULL, env, ctxt = ctxt),
      reason = NULL,
      from = signaller
    )
  } else {
    what <- parse_what(spec, ctxt = ctxt)

    list(
      fn = spec_fn(what$call, ctxt = ctxt),
      arg = spec_arg(what$call, ctxt = ctxt),
      pkg = spec_pkg(what$pkg, env, ctxt = ctxt),
      reason = spec_reason(what$call, ctxt = ctxt),
      from = signaller
    )
  }
}

parse_what <- function(what, ctxt) {
  check_string(what, call = ctxt$call)

  call <- parse_expr(what)

  if (!is_call(call)) {
    what <- as_string(what)
    cli::cli_abort(
      c(
        "{.arg what} must have function call syntax.",
        "",
        " " = "# Good:",
        " " = "{ ctxt$signaller }(\"{what}()\")",
        "",
        " " = "# Bad:",
        " " = "{ ctxt$signaller }(\"{what}\")"
      ),
      call = ctxt$call,
      arg = "what"
    )
  }

  head <- call[[1]]
  if (is_call(head, "::")) {
    pkg <- as_string(head[[2]])
    call[[1]] <- head[[3]]
  } else {
    pkg <- NULL
  }

  list(pkg = pkg, call = call)
}

spec_fn <- function(call, ctxt) {
  fn <- node_car(call)

  if (!is_symbol(fn) && !is_call(fn, "$")) {
    cli::cli_abort(
      "{.arg what} must be a function or method call.",
      call = ctxt$call,
      arg = "what"
    )
  }

  # Deparse so non-syntactic names are backticked
  deparse(fn)
}

spec_arg <- function(call, ctxt) {
  arg <- node_cdr(call)

  if (is_null(arg)) {
    return(NULL)
  }

  if (length(arg) != 1L) {
    fn <- as_label(node_car(call))
    n <- length(arg)
    cli::cli_abort(
      "Function in {.arg what} ({fn}) must have 1 argument, not {n}.",
      call = ctxt$call
    )
  }

  if (is_null(node_tag(arg))) {
    as_string(node_car(arg))
  } else {
    as_string(node_tag(arg))
  }
}

spec_reason <- function(call, ctxt) {
  arg <- node_cdr(call)

  if (is_null(arg)) {
    return(NULL)
  }

  if (is_null(node_tag(arg))) {
    return(NULL)
  }

  if (is_missing(node_car(arg))) {
    return(NULL)
  }

  if (is_string(node_car(arg))) {
    return(node_car(arg))
  }

  fn <- deparse(node_car(call))
  cli::cli_abort(
    c(
      "{.arg what} must contain reason as a string on the RHS of `=`.",
      "",
      " " = "# Good:",
      " " = "{ctxt$signaller}(\"{fn}(arg = 'must be a string')\")",
      "",
      " " = "# Bad:",
      " " = "{ctxt$signaller}(\"{fn}(arg = 42)\")"
    ),
    call = ctxt$call
  )
}

spec_pkg <- function(pkg, env, ctxt) {
  if (!is_null(pkg) || is_null(env)) {
    return(pkg)
  }

  env <- topenv(env)
  if (is_reference(env, global_env())) {
    # Convenient for experimenting interactively
    return(getOption("lifecycle:::calling_package", "<NA>"))
  }

  if (is_namespace(env)) {
    return(ns_env_name(env))
  }

  cli::cli_abort(
    c(
      "Can't detect the package of the deprecated function.",
      "Please mention the namespace:",
      "",
      " " = "# Good:",
      " " = "{ ctxt$signaller }(what = \"namespace::myfunction()\")",
      "",
      " " = "# Bad:",
      " " = "{ ctxt$signaller }(what = \"myfunction()\")",
      ""
    ),
    call = ctxt$call
  )
}

test_that("spec() builds feature data", {
  expect_identical(
    spec("foo()"),
    spec_data(fn = "foo")
  )
  expect_identical(
    spec("pkg::foo()"),
    spec_data(fn = "foo", pkg = "pkg")
  )

  expect_identical(
    spec("foo(bar)"),
    spec_data(fn = "foo", arg = "bar")
  )
  expect_identical(
    spec("foo(bar = )"),
    spec_data(fn = "foo", arg = "bar")
  )
  expect_identical(
    spec("pkg::foo(bar = )"),
    spec_data(fn = "foo", arg = "bar", pkg = "pkg")
  )

  expect_identical(
    spec("foo(bar = 'baz')"),
    spec_data(fn = "foo", arg = "bar", reason = "baz")
  )
  expect_identical(
    spec("pkg::foo(bar = 'baz')"),
    spec_data(fn = "foo", arg = "bar", pkg = "pkg", reason = "baz")
  )
})

test_that("spec() gives useful errors", {
  expect_snapshot(spec(1), error = TRUE)
  expect_snapshot(spec("foo"), error = TRUE)
  expect_snapshot(spec("foo()()"), error = TRUE)
  expect_snapshot(spec("foo(arg = , arg = )"), error = TRUE)
  expect_snapshot(spec("foo(arg = arg)"), error = TRUE)

  e <- new_environment()
  local_options(topLevelEnvironment = e)
  expect_snapshot(spec("foo()", env = e), error = TRUE)
})

test_that("spec() works with methods", {
  expect_identical(
    spec("A$foo()"),
    spec_data(fn = "A$foo")
  )
  expect_identical(
    spec("A$foo(bar = )"),
    spec_data(fn = "A$foo", arg = "bar")
  )

  expect_snapshot(spec("A$foo(bar = 1)"), error = TRUE)
})

expect_lifecycle_defunct <- function(object, ...) {
  expect_error(object, class = "defunctError")
}

expect_no_warning <- function(...) {
  expect_warning(regexp = NA, ...)
}

try2 <- function(expr) {
  cat(paste0("\n", as_label2(substitute(expr)), ":\n\n"))
  cat(catch_cnd(expr, classes = "error")$message, "\n\n")
}

cat_ruler <- function(title) {
  cat(paste0("\n\n", title, "\n", strrep("=", nchar(title)), "\n\n"))
}

spec_data <- function(fn = NULL,
                      arg = NULL,
                      pkg = spec_pkg(NULL, caller_env()),
                      reason = NULL,
                      from = "signal_lifecycle") {
  list(
    fn = fn,
    arg = arg,
    pkg = pkg,
    reason = reason,
    from = from
  )
}

new_callers <- function(deprecated_feature, env = caller_env()) {
  direct <- inject(function(...) (!!deprecated_feature)(...))
  indirect <- inject(function(...) (!!deprecated_feature)(...))

  environment(direct) <- global_env()
  environment(indirect) <- ns_env("base")

  list(direct, indirect)
}

expect_defunct <- function(object, ...) {
  expect_error(object, class = "defunctError")
}

expect_no_warning <- function(...) {
  expect_warning(regexp = NA, ...)
}

try2 <- function(expr) {
  cat(paste0("\n", as_label2(substitute(expr)), ":\n\n"))
  cat(catch_cnd(expr, classes = "error")$message, "\n\n")
}

cat_ruler <- function(title) {
  cat(paste0("\n\n", title, "\n", strrep("=", nchar(title)), "\n\n"))
}

verify_errors <- function(expr) expr

spec_data <- function(fn = NULL, arg = NULL, pkg = NULL, details = NULL) {
  list(
    fn = fn,
    arg = arg,
    pkg = pkg,
    details = details
  )
}

test_that("feature_spec() builds feature data", {
  expect_identical(feature_spec("foo()"), spec_data(fn = "foo"))
  expect_identical(feature_spec("pkg::foo()"), spec_data(fn = "foo", pkg = "pkg"))

  expect_identical(feature_spec("foo(bar = )"), spec_data(fn = "foo", arg = "bar"))
  expect_identical(feature_spec("pkg::foo(bar = )"), spec_data(fn = "foo", arg = "bar", pkg = "pkg"))

  expect_identical(feature_spec("foo(bar = 'baz')"), spec_data(fn = "foo", arg = "bar", details = "baz"))
  expect_identical(feature_spec("pkg::foo(bar = 'baz')"), spec_data(fn = "foo", arg = "bar", pkg = "pkg", details = "baz"))

  verify_errors({
    expect_error(feature_spec("foo"), "")
    expect_error(feature_spec("foo()()"), "")
    expect_error(feature_spec("foo(arg = , arg = )"), "")
    expect_error(feature_spec("foo(arg)"), "")
    expect_error(feature_spec("foo(arg = arg)"), "")
  })
})

test_that("spec.R produces correct error messages", {
  verify_output(test_path("error", "test-spec.txt"), {
    "# feature_spec() builds feature data"
    feature_spec("foo")
    feature_spec("foo()()")
    feature_spec("foo(arg = , arg = )")
    feature_spec("foo(arg)")
    feature_spec("foo(arg = arg)")
  })
})

context("verbosity")

test_that("verbosity option is validated", {
  opt <- with_options(lifecycle_verbosity = NULL, lifecycle_verbosity())
  expect_identical(opt, "default")

  expect_warning(
    with_options(lifecycle_verbosity = NA, lifecycle_verbosity()),
    "must be set to one of"
  )
})

#' Does expression produce lifecycle warnings or errors?
#'
#' @description
#' These functions are equivalent to [testthat::expect_warning()] and
#' [testthat::expect_error()] but check specifically for lifecycle
#' warnings or errors.
#'
#' To test whether a deprecated feature still works without causing a
#' deprecation warning, set the `lifecycle_verbosity` option to
#' `"quiet"`.
#'
#' ```
#' test_that("feature still works", {
#'   withr::local_options(lifecycle_verbosity = "quiet")
#'   expect_true(my_deprecated_function())
#' })
#' ```
#'
#' @param expr Expression that should produce a lifecycle warning or
#'   error.
#' @param regexp Optional regular expression matched against the
#'   expected warning message.
#' @inheritParams testthat::expect_warning
#'
#' @details
#' `expect_deprecated()` sets the [lifecycle_verbosity][verbosity]
#' option to `"warning"` to enforce deprecation warnings which are
#' otherwise only shown once every 8 hours.
#'
#' @export
expect_deprecated <- function(expr, regexp = NULL, ...) {
  local_options(lifecycle_verbosity = "warning")

  if (!is_null(regexp) && is_na(regexp)) {
    abort("`regexp` can't be `NA`.")
  }

  testthat::expect_warning(
    {{ expr }},
    regexp = regexp,
    class = "lifecycle_warning_deprecated",
    ...
  )
}
#' @rdname expect_deprecated
#' @export
expect_defunct <- function(expr) {
  testthat::expect_error(
    {{ expr }},
    class = "lifecycle_error_deprecated"
  )
}

---
title: "Manage lifecycle changes in functions you use"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Manage lifecycle changes in functions you use}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options("lifecycle:::calling_package" = "tibble")
```

The lifecycle package uses warnings to tell you about deprecated functions.
Deprecated functions will be removed in a future release, so it's good practice to eliminate the warnings as soon as you see them.

For example, lets imagine your code uses `tibble::data_frame()`, which was deprecated in favour of `tibble()` in version 1.1.0.
`data_frame()` now looks something like this:

```{r}
data_frame <- function(...) {
  lifecycle::deprecate_warn("1.1.0", "data_frame()", "tibble()")
  tibble::tibble(...)
}
```

That means if you use `data_frame()` in your own code you'll get a warning:

```{r}
df1 <- data_frame(x = 1, y = 2)
#> This warning is displayed once every 8 hours.
#> Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.
df2 <- data_frame(a = "apple", b = "banana")
```

You'll notice that the warning only appears the first time we call it --- lifecycle only notifies you every 8 hours so it's not overly disruptive if you've used a deprecated function in many places.

So how do you track down exactly where the warning came from?
Firstly, you might notice the deprecation warning message includes the advice to call `lifecycle::last_lifecycle_warnings()`.
That'll give you a list of all the deprecation warnings that have happened recently:

```{r, eval = FALSE}
lifecycle::last_lifecycle_warnings()
#> [[1]]
#> <deprecated>
#> message: `data_frame()` was deprecated in tibble 1.1.0.
#> Please use `tibble()` instead.
#> Backtrace:
#>  1. global::data_frame(x = 1)
```

Each warning comes with a back trace that shows you the full sequence of calls that lead to the deprecated function.

Alternatively, if you're ready to spend some time tracking down all your uses of deprecated functions, you can use the `lifecycle_verbosity` option to make deprecated functions warn every time:

```{r}
options(lifecycle_verbosity = "warning")
df1 <- data_frame(x = 1, y = 2)
df2 <- data_frame(a = "apple", b = "banana")
```

Then use `lifecycle::last_lifecycle_warnings()` to track down the source.

Alternatively, if you want to be really strict, you can turn all deprecation warnings into errors, forcing you to deal with them immediately:

```{r, error = TRUE}
options("lifecycle_verbosity" = "error")
df1 <- data_frame(x = 1, y = 2)
```

---
title: "Communicate lifecycle changes in your functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Communicate lifecycle changes in your functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

options(
  # Pretend we're in the lifecycle package
  "lifecycle:::calling_package" = "lifecycle",
  # suppress last_lifecycle_warnings() message by default
  "lifecycle_verbosity" = "warning"
)
```

lifecycle provides a standard way to document the lifecycle stages of functions and arguments, paired with tools to steer users away from deprecated functions.
Before we go in to the details, make sure that you're familiar with the lifecycle stages as described in `vignette("stages")`.

## Basics

lifecycle badges make it easy for users to see the lifecycle stage when reading the documentation.
To use the badges, first call `usethis::use_lifecycle()` to embed the badge images in your package (you only need to do this once), then use `lifecycle::badge()` to insert a badge:

```{r, eval = FALSE}
#' `r lifecycle::badge("experimental")`
#' `r lifecycle::badge("deprecated")`
#' `r lifecycle::badge("superseded")`
```

Deprecated functions also need to advertise their status when run.
lifecycle provides `deprecate_warn()` which takes three main arguments:

-   The first argument, `when`, gives the version number when the deprecation occurred.

-   The second argument, `what`, describes exactly what was deprecated.

-   The third argument, `with`, provides the recommended alternative.

We'll cover the details shortly, but here are a few sample uses:

```{r}
lifecycle::deprecate_warn("1.0.0", "old_fun()", "new_fun()")
lifecycle::deprecate_warn("1.0.0", "fun()", "testthat::fun()")
lifecycle::deprecate_warn("1.0.0", "fun(old_arg)", "fun(new_arg)")
```

(Note that the message includes the package name --- this is automatically discovered from the environment of the calling function so will not work unless the function is called from the package namespace.)

The following sections describe how to use lifecycle badges and functions together to handle a variety of common development tasks.

## Functions

### Deprecate a function

First, add a badge to the the `@description` block[^1].
Briefly describe why the deprecation occurred and what to use instead.

[^1]: We only use an explicit `@description` when the description will be multiple paragraphs, as in these examples.

```{r}
#' Add two numbers
#' 
#' @description
#' `r lifecycle::badge("deprecated")`
#' 
#' This function was deprecated because we realised that it's
#' a special case of the [sum()] function.
```

Next, update the examples to show how to convert from the old usage to the new usage:

```{r}
#' @examples 
#' add_two(1, 2)
#' # ->
#' sum(1, 2)
```

Then add `@keywords internal` to remove the function from the documentation index.
If you use pkgdown, also check that it's no longer listed in `_pkgdown.yml`.
These changes reduce the chance of new users coming across a deprecated function, but don't prevent those who already know about it from referring to the docs.

```{r}
#' @keywords internal
```

You're now done with the docs, and it's time to add a warning when the user calls your function.
Do this by adding call to `deprecate_warn()` on the first line of the function:

```{r}
add_two <- function(x, y) {
  lifecycle::deprecate_warn("1.0.0", "add_two()", "base::sum()")
  x + y
}

add_two(1, 2)
```

`deprecate_warn()` generates user friendly messages for two common deprecation alternatives:

-   Function in same package: `lifecycle::deprecate_warn("1.0.0", "fun_old()", "fun_new()")`

-   Function in another package: `lifecycle::deprecate_warn("1.0.0", "old()", "package::new()")`

For other cases, use the `details` argument to provide your own message to the user:

```{r}
add_two <- function(x, y) {
  lifecycle::deprecate_warn(
    "1.0.0", 
    "add_two()", 
    details = "This function is a special case of sum(); use it instead."
  )
  x + y
}

add_two(1, 2)
```

It's good practice to test that you've correctly implemented the deprecation, testing that the deprecated function still works and that it generates a useful warning.
Using an expectation inside `testthat::expect_snapshot()`[^2] is a convenient way to do this:

[^2]: You can learn more about snapshot testing in `vignette("snapshotting", package = "testthat")`.

```{r, eval = FALSE}
test_that("add_two is deprecated", {
  expect_snapshot({
    x <- add_two(1, 1)
    expect_equal(x, 2)
  })
})
```

If you have existing tests for the deprecated function you can suppress the warning in those tests with the `lifecycle_verbosity` option:

```{r, eval = FALSE }
test_that("add_two returns the sum of its inputs", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_equal(add_two(1, 1), 2)
})
```

And then add a separate test specifically for the deprecation.

```{r, eval = FALSE}
test_that("add_two is deprecated", {
  expect_snapshot(add_two(1, 1))
})
```

### Gradual deprecation

For particularly important functions, you can choose to add two other stages to the deprecation process:

-   `deprecate_soft()` is used before `deprecate_warn()`.
    This function only warns (a) users who try the feature from the global environment and (b) developers who directly use the feature (when running testthat tests).
    There is no warning when the deprecated feature is called indirectly by another package --- the goal is to ensure that warn only the person who has the power to stop using the deprecated feature.

-   `deprecate_stop()` comes after `deprecate_warn()` and generates an error instead of a warning.
    The main benefit over simply removing the function is that the user is informed about the replacement.

If you use these stages you'll also need a process for bumping the deprecation stage for major and minor releases.
We recommend something like this:

1.  Search for `deprecate_stop()` and consider if you're ready to the remove the function completely.

2.  Search for `deprecate_warn()` and replace with `deprecate_stop()`.
    Remove the remaining body of the function and any tests.

3.  Search for `deprecate_soft()` and replace with `deprecate_warn()`.

### Rename a function

To rename a function without breaking existing code, move the implementation to the new function, then call the new function from the old function, along with a deprecation message:

```{r}
#' Add two numbers
#' 
#' @description 
#' `r lifecycle::badge("deprecated")`
#' 
#' `add_two()` was renamed to `number_add()` to create a more
#' consistent API.
#' @keywords internal
#' @export
add_two <- function(foo, bar) {
  lifecycle::deprecate_warn("1.0.0", "add_two()", "number_add()")
  number_add(foo, bar)
}

# documentation goes here...
#' @export
number_add <- function(x, y) {
  x + y
}
```

If you are renaming many functions as part of an API overhaul, it'll often make sense to document all the changes in one file, like <https://rvest.tidyverse.org/reference/rename.html>.

### Supersede a function

Superseding a function is simpler than deprecating it, since you don't need to steer users away from it with a warning.
So all you need to do is add a superseded badge:

```{r}
#' Gather columns into key-value pairs
#'
#' @description
#' `r lifecycle::badge("superseded")`
```

Then describe why the function was superseded, and what the recommended alternative is:

```{r}
#'
#' Development on `gather()` is complete, and for new code we recommend
#' switching to `pivot_longer()`, which is easier to use, more featureful,
#' and still under active development.
#' 
#' In brief,
#' `df %>% gather("key", "value", x, y, z)` is equivalent to
#' `df %>% pivot_longer(c(x, y, z), names_to = "key", values_to = "value")`.
#' See more details in `vignette("pivot")`.
```

The rest of the documentation can stay the same.

If you're willing to live on the bleeding edge of lifecycle, add a call to the experimental `signal_stage()`:

```{r}
gather <- function(data, key = "key", value = "value", ...) {
  lifecycle::signal_stage("superseded", "gather()")
}
```

This signal isn't currently hooked up to any behaviour, but we plan to provide logging and analysis tools in a future release.

### Mark function as experimental

To advertise that a function is experimental and the interface might change in the future, first add an experimental badge to the description:

```{r}
#' @description
#' `r lifecycle::badge("experimental")`
```

If the function is very experimental, you might want to add `@keywords internal` too.

If you're willing to try an experimental lifecycle feature, add a call to `signal_stage()` in the body:

```{r}
cool_function <- function() {
  lifecycle::signal_stage("experimental", "cool_function()")
}
```

This signal isn't currently hooked up to any behaviour, but we plan to provide logging and analysis tools in a future release.

## Arguments

### Deprecate an argument, keeping the existing default

Take this example where we want to deprecate `na.rm` in favour of always making it `TRUE.`

```{r}
add_two <- function(x, y, na.rm = TRUE) {
  sum(x, y, na.rm = na.rm)
}
```

First, add a badge to the argument description:

```{r}
#' @param na.rm `r lifecycle::badge("deprecated")` `na.rm = FALSE` is no
#'   longer supported; this function will always remove missing values
```

And add a deprecation warning if `na.rm` is FALSE.
In this case, there's no replacement to the behaviour, so we instead use `details` to provide a custom message:

```{r}
add_two <- function(x, y, na.rm = TRUE) {
  if (!isTRUE(na.rm)) {
    lifecycle::deprecate_warn(
      when = "1.0.0", 
      what = "add_two(na.rm)",
      details = "Ability to retain missing values will be dropped in next release."
    )
  }
  
  sum(x, y, na.rm = na.rm)
}

add_two(1, NA, na.rm = TRUE)
add_two(1, NA, na.rm = FALSE)
```

### Deprecating an argument, providing a new default

Alternatively, you can change the default value to `lifecycle::deprecated()` to make the deprecation status more obvious from the outside, and use `lifecycle::is_present()` to test whether or not the argument was provided.
Unlike `missing()`, this works for both direct and indirect calls.

```{r}
#' @importFrom lifecycle deprecated
add_two <- function(x, y, na.rm = deprecated()) {
  if (lifecycle::is_present(na.rm)) {
    lifecycle::deprecate_warn(
      when = "1.0.0", 
      what = "add_two(na.rm)",
      details = "Ability to retain missing values will be dropped in next release."
    )
  }
  
  sum(x, y, na.rm = na.rm)
}
```

The chief advantage of this technique is that users will get a warning regardless of what value of `na.rm` they use:

```{r}
add_two(1, NA, na.rm = TRUE)
add_two(1, NA, na.rm = FALSE)
```

### Renaming an argument

You may want to rename an argument if you realise you have made a mistake with the name of an argument.
For example, you've realised that an argument accidentally uses `.` to separate a compound name, instead of `_`.
You'll need to temporarily permit both arguments, generating a deprecation warning when the user supplies the old argument:

```{r}
add_two <- function(x, y, na_rm = TRUE, na.rm = deprecated()) {
  if (lifecycle::is_present(na.rm)) {
    lifecycle::deprecate_warn("1.0.0", "add_two(na.rm)", "add_two(na_rm)")
    na_rm <- na.rm
  }
  
  sum(x, y, na.rm = na_rm)
}

add_two(1, NA, na.rm = TRUE)
```

### Reducing allowed inputs to an argument

To narrow the set of allowed inputs, call `deprecate_warn()` only when the user supplies the previously supported inputs.
Make sure you preserve the previous behaviour:

```{r}
add_two <- function(x, y) {
  if (length(y) != 1) {
    lifecycle::deprecate_warn("1.0.0", "foo(y = 'must be a scalar')")
    y <- sum(y)
  }
  x + y
}

add_two(1, 2)
add_two(1, 1:5)
```

## Anything else

You can wrap `what` and `with` in `I()` to deprecate behaviours not otherwise described above:

```{r}
lifecycle::deprecate_warn(
  when = "1.0.0",
  what = I('Setting the global option "pkg.opt" to "foo"')
)

lifecycle::deprecate_warn(
  when = "1.0.0",
  what = I('The global option "pkg.another_opt"'),
  with = I('"pkg.this_opt"')
)
```

Note that your `what` fragment needs to make sense with "was deprecated ..." added to the end, and your `with` fragment needs to make sense in the sentence "Please use `{with}` instead".

---
title: "Lifecycle stages"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Lifecycle stages}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This vignette describes the four primary stages of the tidyverse lifecycle: stable, deprecated, superseded, and experimental.

![A diagram showing the transitions between the four main stages: experimental can become stable and stable can become deprecated or superseded.](figures/lifecycle.svg){width="75%"}

The lifecycle stages can apply to packages, functions, function arguments, and even specific values of a function argument.
However, you'll mostly see them used to label individual functions, so that's the language we use below.

## Stable

The default development stage is ![stable](figures/lifecycle-stable.svg){style="vertical-align:middle"}.
A function is considered stable when the author is happy with its interface, doesn't see major issues, and is happy to share it with the world.
Because this stage is the default, functions will only be given a stable badge if there's some specific need to draw attention to their status.

Stability is defined in terms of breaking changes.
A **breaking change** is a change that breaks code that uses the function as expected.
In general, breaking changes reduce the set of code that works without error, either by removing a function, removing a function argument, or decreasing the set of valid inputs.
Breaking changes also include changes to output type.
If you imagine all the possible inputs to a function that return a result (and not an error), a breaking change makes the set smaller.

Not all changes that cause your function to stop working are breaking changes.
For example, you might have accidentally relied on a bug.
When the bug is fixed, your code breaks, but this is not a breaking change.
A good way of making your code more robust to this sort of behaviour change is to only use a function for its explicitly intended effects.
For example, using `c()` to concatenate two vectors will not put your code at risk because it clearly is the intended usage of this function.
On the other hand, using `c()` just for the side effect of removing attributes is probably not a good idea.
If you use `c(factor("foo"))` to retrieve the underlying integers of the factor levels, your code is at risk of breaking if `c()` ever implements a factor method.

Stable functions come with two promises related to breaking changes:

-   Breaking changes will be avoided where possible.
    We'll only make breaking changes if we consider the long term benefit of the change to be greater than short term pain of changing existing code.

-   If a breaking change is needed, it will occur gradually, through the deprecation process described next.
    This gives you plenty of time to adjust your code before it starts generating errors.

## Deprecated

A ![Deprecated](figures/lifecycle-deprecated.svg){style="vertical-align:middle"} function has a better alternative available and is scheduled for removal.
When you call a deprecated function, you get a warning telling you what to use instead.
For example, take `tibble::as_data_frame()`:

```{r, eval = FALSE}
df <- tibble::data_frame(x = 1)
#> Warning message:
#> `data_frame()` is deprecated as of tibble 1.1.0.
#> Please use `tibble()` instead.
#> This warning is displayed once every 8 hours.
#> Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated. 
```

The deprecation warning tells you when the function was deprecated (in tibble 1.1.0, released in 2016), and what to use instead (`tibble()`).
To avoid being too annoying, deprecation messages will only appear once per session, and you can find out exactly where they come from by calling `lifecycle::last_lifecycle_warnings()`.
`vignette("manage")` provides more advice on handling deprecation warnings in your code.

Particularly important functions may go through two additional stages of deprecation:

-   **Soft deprecated** comes before deprecated.
    It's a gentler form of deprecation designed to prevent new uses of a function and encourage package developers to move away from it.
    Soft deprecated allows a package to change its interface to encourage package developers to update their code before their users are forced to change.

-   **Defunct** comes after deprecated.
    In most cases, a deprecated function will eventually just be deleted.
    For very important functions, we'll instead make the function defunct, which means that function continues to exist but the deprecation warning turns into an error.
    This is more user-friendly than just removing the function because users will get a clear error message explaining why their code no longer works and how they can fix it.

## Superseded

A softer alternative to deprecation is superseded.
A ![superseded](figures/lifecycle-superseded.svg)[^1] function has a known better alternative, but the function itself is not going away
. A superseded function will not emit a warning (since there's no risk if you keep using it), but the documentation will tell you what we recommend instead
.

[^1]: This stage was previously called retired.

Superseded functions will not receive new features, but will receive any critical bug fixes needed to keep it working.
In some ways a superseded function is actually safer than a stable function because it's guaranteed never to change (for better or for worse).

## Experimental

Some functions are released in an ![experimental](figures/lifecycle-experimental.svg){style="vertical-align:middle"} stage.
Experimental functions are made available so people can try them out and provide feedback, but come with no promises for long term stability.
In particular, the author reserves the right to make breaking changes without a deprecation cycle.
That said, there is some interaction between popularity and stability.
Breaking a popular function, even if clearly labelled as experimental, is likely to cause widespread pain so we'll generally try to avoid it.

In general, you can assume any package with version number less than 1.0.0 is at least somewhat experimental, and it may have major changes in its future.
The most experimental packages only exist on GitHub.
If you're using a non-CRAN package you should plan for an active relationship: when the package changes, you need to be prepared to update your code.

## Superseded stages

We no longer use these stages, but we document them here because we have used them in the past.

### Questioning

Sometimes the author of a function is no longer certain that a function is the optimal approach, but doesn't yet know how to do it better.
These functions can be marked as ![questioning](figures/lifecycle-questioning.svg){style="vertical-align:middle"} to give users a heads up that the author has doubts about the function.
Because knowing that a function is questioning is not very actionable, we no longer use or recommend this stage.

### Maturing

Previously we used as ![maturing](figures/lifecycle-maturing.svg){style="vertical-align:middle"} for functions that lay somewhere between experimental and stable.
We stopped using this stage because, like questioning, it's not clear what actionable information this stage delivers.

---
title: "Usage of the lifecycle package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{lifecycle}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(lifecycle)
```

Use lifecycle to document the status of your exported functions and arguments:

* Choose one of the 7 lifecycle stages a function or argument can be in. You can choose from 4 development stages (experimental, maturing, stable, and questioning) and 3 deprecation stages (soft-deprecated, deprecated, and defunct).

* If the function or argument is deprecated, make sure your users know about by calling `deprecate_soft()`, `deprecate_warn()`, or `deprecate_stop()`. These functions try to be informative without being too verbose, with increasing levels of verbosity as the deprecation stage advances.

* Include the relevant lifecycle badge in your documentation.


### Stages

The lifecycle stages for functions and arguments are summarised in the figure below. They're designed to closely mirror the lifecycle stages for [packages](https://www.tidyverse.org/lifecycle/).

<img src="figures/lifecycle.svg" width="75%" />

There are two development stages.

* <img src="figures/lifecycle-experimental.svg" alt = "Experimental" style="vertical-align:middle" /> This is a new feature that is in the very early stage of development. It is exported so users can start to use it and report feedback, but its interface and/or behaviour is likely to change in the future. It is generally best to avoid depending on experimental features.

* <img src="figures/lifecycle-maturing.svg" alt = "Maturing" style="vertical-align:middle" /> The interface and behaviour of a maturing feature has been roughed out, but finer details are likely to change. It still needs more feedback to find the optimal API.

The probable end state of most functions is _stable_:

* <img src="figures/lifecycle-stable.svg" alt = "Stable" style="vertical-align:middle" /> A feature is considered stable when the author is happy with its interface and behaviour. Major changes are unlikely, and breaking changes will occur gradually, through a deprecation process.

Sometimes we are no longer certain that a feature follows or implements the right approach. In this case, we mark it as _questioning_. It can then either become _superseded_ once we have implemented an alternative, or _deprecated_. The difference between superseded and deprecated is that a superseded feature is maintained indefinitely for backward compatibility. Alternatively, we may realise that a questioning feature is actually important, and move it back to stable. User feedback is usually important for taking these decisions and always welcome.

1. <img src="figures/lifecycle-questioning.svg" alt = "Questioning" style="vertical-align:middle" /> The author is no longer convinced that the feature is the optimal approach. However, there are no recommended alternatives yet.

1. <img src="figures/lifecycle-superseded.svg" alt = "Superseded" style="vertical-align:middle" /> (Previously called retired) A superseded feature is no longer under active development, and a known better alternative is available. However, it is indefinitely kept in the package for backward compatibility. The author will only make the necessary changes to ensure that the function continues working. No new features will be added, and only critical bugs will be fixed.

The deprecation lifecycle is divided into three stages in order to give time to users to switch to an alternative.

1. <img src="figures/lifecycle-soft-deprecated.svg" alt = "Soft deprecated" style="vertical-align:middle" /> The author is no longer happy with a feature because they consider it sub-optimal compared to some other approach, or simply because they no longer have the time to maintain it. A soft-deprecated feature can still be used without hassle, but users should consider switching to an alternative approach.

1. <img src="figures/lifecycle-deprecated.svg" alt = "Deprecated" style="vertical-align:middle" /> The feature is likely to be discontinued in the next major release. Users should switch to an alternative approach as soon as possible.

1. <img src="figures/lifecycle-defunct.svg" alt = "Defunct" style="vertical-align:middle" /> The feature can no longer be used. A defunct function is still exported, and a defunct argument is still part of the signature. This way an informative error can be thrown.

Finally, when a feature is no longer exposed or mentioned in the released version of the package, it is said to be _archived_.


### Badges {#rd-badges}

Make sure your users know what stage a feature is by adding badges in the help topics of your functions.

<img src="figures/example-badge.png" align="center" alt = "badge" />

*   Call `usethis::use_lifecycle()` to import the badges in your package.

*   Use the `lifecycle` Rd macro to insert a badge:

    ```
    #' \lifecycle{experimental}
    #' \lifecycle{soft-deprecated}
    ```

    This badge renders as text in non-HTML documentation. To document the status of a whole function, a good place to include the badge is at the top of the `@description` block. To document an argument, you can put the badge in the argument description.

*   For functions in development, you typically don't need to advertise the status if it is the same as the package as a whole. For instance, if your package is [maturing](https://www.tidyverse.org/lifecycle/#maturing), only signal functions in the experimental, stable, and questioning stages.


### Verbosity of deprecation

lifecycle offers three levels of verbosity corresponding to the three deprecation stages.

*   __Soft deprecation__: At this stage, call `deprecate_soft()` to start warning users about the deprecation in the least disruptive way.

    This function only warns (a) users who try the feature from the global workspace, and (b) developers who directly use the feature, when they run unit tests with testthat. No warning is issued outside of unit tests, or when the deprecated feature is called from another package then ther own.

    When a warning does get issued, users only see it once every 8 hours rather than at each invokation.

*   __Deprecation__: At this stage, call `deprecate_warn()` to warn unconditionally about the deprecated feature. The warning is issued only once every 8 hours.

*   __Defunct__: The feature is discontinued. Call `deprecate_stop()` to fail with an error.


### Deprecating functions

These functions take the version number starting from which the feature is considered deprecated (it should remain the same across all deprecation stages), and a feature descriptor:

```{r}
deprecate_warn("1.0.0", "mypkg::foo()")
```

You can optionally provide a replacement:

```{r}
deprecate_warn("1.0.0", "mypkg::foo()", "new()")
```

For the purpose of these examples we explicitly mentioned the namespace with `mypkg::`, however you can typically omit it because lifecycle infers the namespace from the calling environment. Specifying the namespace is mostly useful when the replacement is implemented in a different package.

```{r}
# The new replacement
foobar_adder <- function(foo, bar) {
  foo + bar
}

# The old function still exported for compatibility
foobaz_adder <- function(foo, bar) {
  deprecate_warn("1.0.0", "foobaz_adder()", "foobar_adder()")
  foobar_adder(foo, bar)
}
```


### Deprecating arguments

The syntax for deprecating argument is based on the syntax for deprecating functions:

```{r}
deprecate_warn("1.0.0", "mypkg::foo(arg = )")

deprecate_warn("1.0.0", "mypkg::foo(arg = )", "mypkg::foo(new = )")
```

An argument can be partially deprecated by disallowing certain input types:

```{r}
deprecate_warn("1.0.0", "mypkg::foo(arg = 'must be a scalar integer')")
```

lifecycle also provides the `deprecated()` sentinel to use as default argument. This provides self-documentation for your users and makes it possible for external tools to determine which arguments are deprecated. Test whether the argument was supplied by the caller with `lifecycle::is_present()`:

```{r}
foobar_adder <- function(foo, bar, baz = deprecated()) {
  # Check if user has supplied `baz` instead of `bar`
  if (lifecycle::is_present(baz)) {

    # Signal the deprecation to the user
    deprecate_warn("1.0.0", "foobar_adder(baz = )", "foobar_adder(bar = )")

    # Deal with the deprecated argument for compatibility
    bar <- baz
  }

  foo + bar
}
```


### Workflow

#### Where do these deprecation warnings come from?

Call `lifecycle::last_warnings()` to see backtraces for all the deprecation warnings that were issued during the last top-level command.


#### Bumping deprecation stage

Some manual search and replace is needed to bump the status of deprecated features. We recommend starting with defunct features and work your way up:

1. Search for `deprecate_stop()` and remove the feature from the package. The feature is now archived.

1. Search for `deprecate_warn()` and replace with `deprecate_stop()`.

1. Search for `deprecate_soft()` and replace with `deprecate_warn()`.

1. Call `deprecate_soft()` from newly deprecated functions.

Don't forget to update the badges in the documentation topics.


#### Find out what deprecated features you rely on

Test whether your package depends on deprecated features directly or indirectly by setting the verbosity option in the `tests/testthat.R` file just before `test_check()` is called:

```{r, eval = FALSE}
library(testthat)
library(mypackage)

options(lifecycle_verbosity = "error")
test_check("mypackage")
```

This forces all deprecated features to fail. You can also set the relevant options manually to force warnings or errors in your session:

```{r, eval = FALSE}
# Force silence
options(lifecycle_verbosity = "quiet")

# Force warnings
options(lifecycle_verbosity = "warning")

# Force errors
options(lifecycle_verbosity = "error")
```

Forcing warnings can be useful in conjuction with `last_warnings()`, which prints backtraces for all the deprecation warnings issued during the last top-level command.


#### Test deprecated features

Test whether a deprecated feature still works by setting `lifecycle_verbosity` to `"quiet"`:

```{r, eval = FALSE}
test_that("`baz` argument of `foobar_adder()` still works", {
  withr::local_options(list(lifecycle_verbosity = "quiet"))
  foobar_adder(1, baz = 2)
})
```

You can also set up verbosity for a whole testthat file within `setup()` and `teardown()` blocks:

```{r, eval = FALSE}
setup(options(lifecycle_verbosity = "quiet"))
teardown(options(lifecycle_verbosity = NULL))
```

Test that a feature is correctly deprecated with
`expect_deprecated()` or `expect_defunct()`:

```{r, eval = FALSE}
test_that("`baz` argument of `foobar_adder()` is deprecated", {
  expect_deprecated(foobar_adder(1, baz = 2))
})

test_that("`foo()` is defunct", {
  expect_defunct(foo())
})
```

More control over verbosity can be exercised with the
`lifecycle_verbosity` option. See `?verbosity`.

# deprecation warning is displayed with backtrace

    Code
      last_lifecycle_warnings()
    Output
      [[1]]
      <warning/lifecycle_warning_deprecated>
      Warning:
      `trace()` was deprecated in lifecycle 1.0.0.
      ---
      Backtrace:
           x
        1. +-lifecycle::expect_deprecated(f())
        2. | \-testthat::expect_warning(...)
        3. |   \-testthat:::expect_condition_matching(...)
        4. |     \-testthat:::quasi_capture(...)
        5. |       +-testthat (local) .capture(...)
        6. |       | \-base::withCallingHandlers(...)
        7. |       \-rlang::eval_bare(quo_get_expr(.quo), quo_get_env(.quo))
        8. \-lifecycle (local) f()
        9.   \-lifecycle (local) g()
       10.     \-lifecycle (local) h()
      

# spec() gives useful errors

    Code
      spec(1)
    Condition
      Error:
      ! `what` must be a single string, not a number.

---

    Code
      spec("foo")
    Condition
      Error:
      ! `what` must have function call syntax.
      
        # Good:
        signal_lifecycle("foo()")
      
        # Bad:
        signal_lifecycle("foo")

---

    Code
      spec("foo()()")
    Condition
      Error:
      ! `what` must be a function or method call.

---

    Code
      spec("foo(arg = , arg = )")
    Condition
      Error:
      ! Function in `what` (foo) must have 1 argument, not 2.

---

    Code
      spec("foo(arg = arg)")
    Condition
      Error:
      ! `what` must contain reason as a string on the RHS of `=`.
      
        # Good:
        signal_lifecycle("foo(arg = 'must be a string')")
      
        # Bad:
        signal_lifecycle("foo(arg = 42)")

---

    Code
      spec("foo()", env = e)
    Condition
      Error:
      ! Can't detect the package of the deprecated function.
      Please mention the namespace:
      
        # Good:
        signal_lifecycle(what = "namespace::myfunction()")
      
        # Bad:
        signal_lifecycle(what = "myfunction()")

# spec() works with methods

    Code
      spec("A$foo(bar = 1)")
    Condition
      Error:
      ! `what` must contain reason as a string on the RHS of `=`.
      
        # Good:
        signal_lifecycle("A$foo(arg = 'must be a string')")
      
        # Bad:
        signal_lifecycle("A$foo(arg = 42)")

# default deprecations behave as expected

    Code
      (expect_warning(direct(), class = "lifecycle_warning_deprecated"))
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      i Please use `bar()` instead.

---

    Code
      (expect_defunct(deprecate_stop("1.0.0", "foo()")))
    Output
      <error/lifecycle_error_deprecated>
      Error:
      ! `foo()` was deprecated in lifecycle 1.0.0 and is now defunct.

# deprecate_warn() only warns repeatedly if always = TRUE

    Code
      direct()
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
    Code
      direct()
      indirect()
      indirect()

---

    Code
      direct(always = TRUE)
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
    Code
      direct(always = TRUE)
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
    Code
      indirect(always = TRUE)
      indirect(always = TRUE)

# indirect usage recommends contacting authors

    Code
      indirect(id = "test_base")
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      i The deprecated feature was likely used in the base package.
        Please report the issue to the authors.
    Code
      indirect_rlang(id = "test_rlang")
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      i The deprecated feature was likely used in the rlang package.
        Please report the issue at <https://github.com/r-lib/rlang/issues>.

# what deprecation messages are readable

    Code
      cat_line(lifecycle_message("1.0.0", "foo()"))
    Output
      `foo()` was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "foo()", signaller = "deprecate_stop"))
    Output
      `foo()` was deprecated in base 1.0.0 and is now defunct.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)"))
    Output
      The `arg` argument of `foo()` is deprecated as of base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)", signaller = "deprecate_stop"))
    Output
      The `arg` argument of `foo()` was deprecated in base 1.0.0 and is now defunct.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas")))
    Output
      Use of bananas was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas"), signaller = "deprecate_stop"))
    Output
      Use of bananas was deprecated in base 1.0.0 and is now defunct.

# replace deprecation messages are readable

    Code
      cat_line(lifecycle_message("1.0.0", "foo()", "package::bar()"))
    Output
      `foo()` was deprecated in base 1.0.0.
      Please use `package::bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo()", "bar()"))
    Output
      `foo()` was deprecated in base 1.0.0.
      Please use `bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg1)", "foo(arg2)"))
    Output
      The `arg1` argument of `foo()` is deprecated as of base 1.0.0.
      Please use the `arg2` argument instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)", "bar(arg)"))
    Output
      The `arg` argument of `foo()` is deprecated as of base 1.0.0.
      Please use the `arg` argument of `bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas"), I("apples")))
    Output
      Use of bananas was deprecated in base 1.0.0.
      Please use apples instead.

# unusual names are handled gracefully

    Code
      cat_line(lifecycle_message("1.0.0", "`foo-fy`(`qu-ux` = )"))
    Output
      The `qu-ux` argument of `foo-fy()` is deprecated as of base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "`foo<-`()"))
    Output
      `foo<-()` was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "`+`()"))
    Output
      `+()` was deprecated in base 1.0.0.

# details uses an info bullet by default

    Code
      deprecate_warn("1.0.0", "foo()", details = "Please do that instead.", id = "test")
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      i Please do that instead.
      i The deprecated feature was likely used in the base package.
        Please report the issue to the authors.

---

    Code
      deprecate_warn("1.0.0", "foo()", details = c("Please do that instead.",
        "Also know that."), id = "test")
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      i Please do that instead.
      i Also know that.
      i The deprecated feature was likely used in the base package.
        Please report the issue to the authors.

# can use bullets in details 

    Code
      deprecate_warn("1.0.0", "foo()", details = c("Unnamed", i = "Informative", x = "Error"),
      id = "test")
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      Unnamed
      i Informative
      x Error
      i The deprecated feature was likely used in the base package.
        Please report the issue to the authors.

# checks input types

    Code
      lifecycle_message(1)
    Condition
      Error:
      ! `when` must be a single string, not a number.

---

    Code
      lifecycle_message("1", details = 1)
    Condition
      Error:
      ! `details` must be a character vector, not a number.

# needs_warning works as expected

    Code
      needs_warning(1)
    Condition
      Error:
      ! `id` must be a single string, not a number.

---

    Expected `POSIXct` value in `needs_warning()`.
    i This is an internal error that was detected in the lifecycle package.
      Please report it at <https://github.com/r-lib/lifecycle/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# badge doesn't change unexpected

    Code
      cat(badge("deprecated"))
    Output
      \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

---

    Code
      cat(badge("experimental"))
    Output
      \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

---

    Code
      cat(badge("unknown"))
    Condition
      Error in `badge()`:
      ! `stage` must be one of "experimental", "stable", "superseded", or "deprecated", not "unknown".

# deprecation warnings are not displayed again

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      This warning is displayed once per session.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

---

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

---

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

# deprecate_warn() only warns repeatedly if always = TRUE

    Code
      deprecate()
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
    Code
      deprecate()

---

    Code
      deprecate(always = TRUE)
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
    Code
      deprecate(always = TRUE)
    Condition
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.

# what deprecation messages are readable

    Code
      cat_line(lifecycle_message("1.0.0", "foo()"))
    Output
      `foo()` was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "foo()", signaller = "deprecate_stop"))
    Output
      `foo()` was deprecated in base 1.0.0 and is now defunct.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)"))
    Output
      The `arg` argument of `foo()` is deprecated as of base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)", signaller = "deprecate_stop"))
    Output
      The `arg` argument of `foo()` was deprecated in base 1.0.0 and is now defunct.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas")))
    Output
      Use of bananas was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas"), signaller = "deprecate_stop"))
    Output
      Use of bananas was deprecated in base 1.0.0 and is now defunct.

# replace deprecation messages are readable

    Code
      cat_line(lifecycle_message("1.0.0", "foo()", "package::bar()"))
    Output
      `foo()` was deprecated in base 1.0.0.
      Please use `package::bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo()", "bar()"))
    Output
      `foo()` was deprecated in base 1.0.0.
      Please use `bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg1)", "foo(arg2)"))
    Output
      The `arg1` argument of `foo()` is deprecated as of base 1.0.0.
      Please use the `arg2` argument instead.
    Code
      cat_line(lifecycle_message("1.0.0", "foo(arg)", "bar(arg)"))
    Output
      The `arg` argument of `foo()` is deprecated as of base 1.0.0.
      Please use the `arg` argument of `bar()` instead.
    Code
      cat_line(lifecycle_message("1.0.0", I("Use of bananas"), I("apples")))
    Output
      Use of bananas was deprecated in base 1.0.0.
      Please use apples instead.

# unusual names are handled gracefully

    Code
      cat_line(lifecycle_message("1.0.0", "`foo-fy`(`qu-ux` = )"))
    Output
      The `qu-ux` argument of `foo-fy` is deprecated as of base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "`foo<-`()"))
    Output
      `foo<-` was deprecated in base 1.0.0.
    Code
      cat_line(lifecycle_message("1.0.0", "`+`()"))
    Output
      `+` was deprecated in base 1.0.0.

# can use bullets in details 

    Code
      cat_line(lifecycle_message("1.0.0", "foo()", details = c("Unnamed", i = "Informative",
        x = "Error"), signaller = "deprecate_stop"))
    Output
      `foo()` was deprecated in base 1.0.0 and is now defunct.
      Unnamed
      i Informative
      x Error

# checks input types

    Code
      lifecycle_message(1)
    Condition
      Error in `lifecycle_abort()`:
      ! Internal error in lifecycle: `when` must be a string

---

    Code
      lifecycle_message("1", details = 1)
    Condition
      Error in `lifecycle_abort()`:
      ! Internal error in lifecycle: `details` must be a character vector

# needs_warning works as expected

    Code
      needs_warning(1)
    Condition
      Error in `lifecycle_abort()`:
      ! Internal error in lifecycle: `id` must be a string

---

    Internal error in lifecycle: Expected `POSIXct` value in `needs_warning()`.

# `signal_experimental()` and `signal_superseded()` are deprecated

    Code
      signal_experimental("1.1.0", "foo()")
    Condition
      Warning:
      `signal_experimental()` was deprecated in lifecycle 1.1.0.
      i Please use `signal_stage()` instead.

---

    Code
      signal_superseded("1.1.0", "foo()")
    Condition
      Warning:
      `signal_superseded()` was deprecated in lifecycle 1.1.0.
      i Please use `signal_stage()` instead.

# deprecation warning is displayed with backtrace

    Code
      last_lifecycle_warnings()
    Output
      [[1]]
      <warning/lifecycle_warning_deprecated>
      Warning:
      `trace()` was deprecated in lifecycle 1.0.0.
      ---
      Backtrace:
        1. lifecycle::expect_deprecated(f())
        8. lifecycle (local) f()
        9. lifecycle (local) g()
       10. lifecycle (local) h()
      

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifecycle-package.R
\docType{package}
\name{lifecycle-package}
\alias{lifecycle}
\alias{lifecycle-package}
\title{lifecycle: Manage the Life Cycle of your Package Functions}
\description{
Manage the life cycle of your exported functions with shared conventions, documentation badges, and user-friendly deprecation warnings.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://lifecycle.r-lib.org/}
  \item \url{https://github.com/r-lib/lifecycle}
  \item Report bugs at \url{https://github.com/r-lib/lifecycle/issues}
}

}
\author{
\strong{Maintainer}: Lionel Henry \email{lionel@rstudio.com}

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@rstudio.com} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})
}

Other contributors:
\itemize{
  \item RStudio [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signal.R
\name{signal_stage}
\alias{signal_stage}
\title{Signal other experimental or superseded features}
\usage{
signal_stage(stage, what, with = NULL, env = deprecated())
}
\arguments{
\item{stage}{Life cycle stage, either \code{"experimental"} or \code{"superseded"}.}

\item{what}{String describing what feature the stage applies too, using the
same syntax as \code{\link[=deprecate_warn]{deprecate_warn()}}.}

\item{with}{An optional string giving a recommended replacement for a
superseded function.}

\item{env}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{signal_stage()} allows you to signal life cycle stages other than
deprecation (for which you should use \code{\link[=deprecate_warn]{deprecate_warn()}} and friends). There
is no behaviour associated with this signal, it is currently purely a way to
express intent at the call site. In the future, we hope to replace this with
a standardized call to \code{base::declare()}.
}
\examples{
foofy <- function(x, y, z) {
  signal_stage("experimental", "foofy()")
  x + y / z
}
foofy(1, 2, 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lint.R
\name{pkg_lifecycle_statuses}
\alias{pkg_lifecycle_statuses}
\alias{lint_lifecycle}
\alias{lint_tidyverse_lifecycle}
\title{Lint usages of functions that have a non-stable life cycle.}
\usage{
pkg_lifecycle_statuses(
  package,
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired")
)

lint_lifecycle(
  packages,
  path = ".",
  pattern = "[.][Rr](md)?",
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired")
)

lint_tidyverse_lifecycle(
  path = ".",
  pattern = "[.][Rr](md)?",
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired")
)
}
\arguments{
\item{package}{The name of an installed package.}

\item{which}{The lifecycle statuses to retrieve.
Include \code{NA} if you want to include functions without a specified lifecycle
status in the results.}

\item{packages}{One or more installed packages to query for lifecycle statuses.}

\item{path}{The directory path to the files you want to search.}

\item{pattern}{Any files matching this pattern will be searched. The default
searches any files ending in \code{.R} or \code{.Rmd}.}
}
\description{
\itemize{
\item \code{lint_lifecycle} dynamically queries the package documentation for packages
in \code{packages} for lifecycle annotations and then searches the directory in
\code{path} for usages of those functions.
\item \code{lint_tidyverse_lifecycle} is a convenience function to call \code{lint_lifecycle}
for all the packages in the tidyverse.
\item \code{pkg_lifecycle_statuses} returns a data frame of functions with lifecycle
annotations for an installed package.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/warning.R
\name{last_warnings}
\alias{last_warnings}
\alias{last_warning}
\title{Display last deprecation warnings}
\usage{
last_warnings()

last_warning()
}
\description{
Call these helpers to see the last deprecation warnings along with
their backtrace:
\itemize{
\item \code{last_warnings()} returns a list of all warnings that occurred
during the last top-level R command.
\item \code{last_warning()} returns only the last.
}

If you call these in the console, these warnings are printed with a
backtrace. Pass the \code{simplify} argument to control the verbosity of
the backtrace. It supports one of \code{"branch"} (the default),
\code{"collapse"}, and \code{"none"} (in increasing order of verbosity).
}
\examples{
# These examples are not run because `last_warnings()` does not
# work well within knitr and pkgdown
\dontrun{

f <- function() invisible(g())
g <- function() list(h(), i())
h <- function() deprecate_warn("1.0.0", "this()")
i <- function() deprecate_warn("1.0.0", "that()")
f()

# Print all the warnings that occurred during the last command:
last_warnings()

# Print only the last one:
last_warning()


# By default, the backtraces are printed in their simplified form.
# Use `simplify` to control the verbosity:
print(last_warnings(), simplify = "none")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbosity.R
\name{verbosity}
\alias{verbosity}
\title{Control the verbosity of deprecation signals}
\description{
There are 3 levels of verbosity for deprecated functions: silence,
warning, and error. Since the lifecycle package avoids disruptive
warnings, the default level of verbosity depends on the lifecycle
stage of the deprecated function, on the context of the caller
(global environment or testthat unit tests cause more warnings),
and whether the warning was already issued (see the help for
\link[=deprecate_soft]{deprecation functions}).

You can control the level of verbosity with the global option
\code{lifecycle_verbosity}. It can be set to:
\itemize{
\item \code{"quiet"} to suppress all deprecation messages.
\item \code{"default"} or \code{NULL} to warn once every 8 hours.
\item \code{"warning"} to warn every time.
\item \code{"error"} to error instead of warning.
}

Note that functions calling \code{\link[=deprecate_stop]{deprecate_stop()}} invariably throw
errors.
}
\examples{
if (rlang::is_installed("testthat")) {
  library(testthat)

  mytool <- function() {
    deprecate_soft("1.0.0", "mytool()")
    10 * 10
  }

  # Forcing the verbosity level is useful for unit testing. You can
  # force errors to test that the function is indeed deprecated:
  test_that("mytool is deprecated", {
    rlang::local_options(lifecycle_verbosity = "error")
    expect_error(mytool(), class = "defunctError")
  })

  # Or you can enforce silence to safely test that the function
  # still works:
  test_that("mytool still works", {
    rlang::local_options(lifecycle_verbosity = "quiet")
    expect_equal(mytool(), 100)
  })
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signal.R
\name{signal_experimental}
\alias{signal_experimental}
\alias{signal_superseded}
\title{Deprecated funtions for signalling experimental and lifecycle stages}
\usage{
signal_experimental(when, what, env = caller_env())

signal_superseded(when, what, env = caller_env())
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Please use \code{\link[=signal_stage]{signal_stage()}} instead
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badge.R
\name{badge}
\alias{badge}
\title{Embed a lifecycle badge in documentation}
\usage{
badge(stage)
}
\arguments{
\item{stage}{A lifecycle stage as a string. Must be one of
\code{"experimental"}, \code{"stable"}, \code{"superseded"}, or \code{"deprecated"}.}
}
\value{
An \code{Rd} expression describing the lifecycle stage.
}
\description{
To include lifecycle badges in your documentation:
\enumerate{
\item Call \code{usethis::use_lifecycle()} to copy the badge images into the
\verb{man/} folder of your package.
\item Call \code{lifecycle::badge()} inside R backticks to insert a
lifecycle badge:

\if{html}{\out{<div class="sourceCode">}}\preformatted{#' `r lifecycle::badge("experimental")`
#' `r lifecycle::badge("deprecated")`
#' `r lifecycle::badge("superseded")`
}\if{html}{\out{</div>}}

If the deprecated feature is a function, a good place for this
badge is at the top of the topic description. If it is an argument,
you can put the badge in the argument description.
}

The badge is displayed as an image in the HTML version of the
documentation and as text otherwise.

\code{lifecycle::badge()} is run by roxygen at build time so you don't need
to add lifecycle to \verb{Imports:} just to use the badges. However, it's still
good practice to add to \verb{Suggests:} so that it will be available to
package developers.
}
\section{Badges}{

\itemize{
\item \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} \code{lifecycle::badge("experimental")}
\item \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}} \code{lifecycle::badge("stable")}
\item \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}} \code{lifecycle::badge("superseded")}
\item \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} \code{lifecycle::badge("deprecated")}
}

The meaning of these stages is described in
\code{vignette("stages")}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/warning.R
\name{last_lifecycle_warnings}
\alias{last_lifecycle_warnings}
\title{Display last deprecation warnings}
\usage{
last_lifecycle_warnings()
}
\description{
\code{last_lifecycle_warnings()} returns a list of all warnings that
occurred during the last top-level R command, along with a
backtrace.

Use \code{print(last_lifecycle_warnings(), simplify = level)} to control
the verbosity of the backtrace. The \code{simplify} argument supports
one of \code{"branch"} (the default), \code{"collapse"}, and \code{"none"} (in
increasing order of verbosity).
}
\examples{
# These examples are not run because `last_lifecycle_warnings()` does not
# work well within knitr and pkgdown
\dontrun{

f <- function() invisible(g())
g <- function() list(h(), i())
h <- function() deprecate_warn("1.0.0", "this()")
i <- function() deprecate_warn("1.0.0", "that()")
f()

# Print all the warnings that occurred during the last command:
last_lifecycle_warnings()


# By default, the backtraces are printed in their simplified form.
# Use `simplify` to control the verbosity:
print(last_lifecycle_warnings(), simplify = "none")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expect.R
\name{expect_deprecated}
\alias{expect_deprecated}
\alias{expect_defunct}
\title{Does expression produce lifecycle warnings or errors?}
\usage{
expect_deprecated(expr, regexp = NULL, ...)

expect_defunct(expr)
}
\arguments{
\item{expr}{Expression that should produce a lifecycle warning or
error.}

\item{regexp}{Optional regular expression matched against the
expected warning message.}

\item{...}{
  Arguments passed on to \code{\link[testthat:expect_match]{expect_match}}
  \describe{
    \item{\code{fixed}}{If \code{TRUE}, treats \code{regexp} as a string to be matched exactly
(not a regular expressions). Overrides \code{perl}.}
    \item{\code{perl}}{logical.  Should Perl-compatible regexps be used?}
  }}
}
\description{
These functions are equivalent to \code{\link[testthat:expect_error]{testthat::expect_warning()}} and
\code{\link[testthat:expect_error]{testthat::expect_error()}} but check specifically for lifecycle
warnings or errors.

To test whether a deprecated feature still works without causing a
deprecation warning, set the \code{lifecycle_verbosity} option to
\code{"quiet"}.

\if{html}{\out{<div class="sourceCode">}}\preformatted{test_that("feature still works", \{
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_true(my_deprecated_function())
\})
}\if{html}{\out{</div>}}
}
\details{
\code{expect_deprecated()} sets the \link[=verbosity]{lifecycle_verbosity}
option to \code{"warning"} to enforce deprecation warnings which are
otherwise only shown once every 8 hours.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lint.R
\name{pkg_lifecycle_statuses}
\alias{pkg_lifecycle_statuses}
\alias{lint_lifecycle}
\alias{lint_tidyverse_lifecycle}
\alias{lifecycle_linter}
\title{Lint usages of functions that have a non-stable life cycle.}
\usage{
pkg_lifecycle_statuses(
  package,
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired")
)

lint_lifecycle(
  packages,
  path = ".",
  pattern = "(?i)[.](r|rmd|qmd|rnw|rhtml|rrst|rtex|rtxt)$",
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired"),
  symbol_is_undesirable = FALSE
)

lint_tidyverse_lifecycle(
  path = ".",
  pattern = "(?i)[.](r|rmd|qmd|rnw|rhtml|rrst|rtex|rtxt)$",
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired"),
  symbol_is_undesirable = FALSE
)

lifecycle_linter(
  packages = tidyverse::tidyverse_packages(),
  which = c("superseded", "deprecated", "questioning", "defunct", "experimental",
    "soft-deprecated", "retired"),
  symbol_is_undesirable = FALSE
)
}
\arguments{
\item{package}{The name of an installed package.}

\item{which}{Vector of lifecycle statuses to lint.}

\item{packages}{One or more installed packages to query for lifecycle statuses.}

\item{path}{The directory path to the files you want to search.}

\item{pattern}{Any files matching this pattern will be searched. The default
searches any files ending in \code{.R} or \code{.Rmd}.}

\item{symbol_is_undesirable}{Also lint symbol usages, e.g. \code{lapply(x, is_na)}?}
}
\description{
\itemize{
\item \code{lifecycle_linter()} creates a linter for lifecycle annotations which can be
included in a \code{.lintr} configuration if \code{lintr} is used directly.
\item \code{lint_lifecycle()} dynamically queries the package documentation for packages
in \code{packages} for lifecycle annotations and then searches the directory in
\code{path} for usages of those functions.
\item \code{lint_tidyverse_lifecycle()} is a convenience function to call \code{lint_lifecycle()}
for all the packages in the tidyverse.
\item \code{pkg_lifecycle_statuses()} returns a data frame of functions with lifecycle
annotations for an installed package.
}
}
\examples{
lintr::lint(
  text = "is_na(x)",
  linters = lifecycle_linter(packages = "rlang")
)
lintr::lint(
  text = "lapply(x, is_na)",
  linters = lifecycle_linter(packages = "rlang",
  symbol_is_undesirable = TRUE)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecate.R
\name{deprecate_soft}
\alias{deprecate_soft}
\alias{deprecate_warn}
\alias{deprecate_stop}
\title{Deprecate functions and arguments}
\usage{
deprecate_soft(
  when,
  what,
  with = NULL,
  details = NULL,
  id = NULL,
  env = caller_env(),
  user_env = caller_env(2)
)

deprecate_warn(
  when,
  what,
  with = NULL,
  details = NULL,
  id = NULL,
  always = FALSE,
  env = caller_env(),
  user_env = caller_env(2)
)

deprecate_stop(when, what, with = NULL, details = NULL, env = caller_env())
}
\arguments{
\item{when}{A string giving the version when the behaviour was deprecated.}

\item{what}{A string describing what is deprecated:
\itemize{
\item Deprecate a whole function with \code{"foo()"}.
\item Deprecate an argument with \code{"foo(arg)"}.
\item Partially deprecate an argument with
\code{"foo(arg = 'must be a scalar integer')"}.
\item Deprecate anything else with a custom message by wrapping it in \code{I()}.
}

You can optionally supply the namespace: \code{"ns::foo()"}, but this is
usually not needed as it will be inferred from the caller environment.}

\item{with}{An optional string giving a recommended replacement for the
deprecated behaviour. This takes the same form as \code{what}.}

\item{details}{In most cases the deprecation message can be
automatically generated from \code{with}. When it can't, use \code{details}
to provide a hand-written message.

\code{details} can either be a single string or a character vector,
which will be converted to a \link[cli:cli_bullets]{bulleted list}.
By default, info bullets are used. Provide a named vectors to
override.}

\item{id}{The id of the deprecation. A warning is issued only once
for each \code{id}. Defaults to the generated message, but you should
give a unique ID when the message in \code{details} is built
programmatically and depends on inputs, or when you'd like to
deprecate multiple functions but warn only once for all of them.}

\item{env, user_env}{Pair of environments that define where \verb{deprecate_*()}
was called (used to determine the package name) and where the function
called the deprecating function was called (used to determine if
\code{deprecate_soft()} should message).

These are only needed if you're calling \verb{deprecate_*()} from an internal
helper, in which case you should forward \code{env = caller_env()} and
\code{user_env = caller_env(2)}.}

\item{always}{If \code{FALSE}, the default, will warn every 8 hours.  If
\code{TRUE}, will always warn in direct usages. Indirect usages keep
warning every 8 hours to avoid disrupting users who can't fix the
issue. Only use \code{always = TRUE} after at least one release with
the default.}
}
\value{
\code{NULL}, invisibly.
}
\description{
These functions provide three levels of verbosity for deprecated
functions. Learn how to use them in \code{vignette("communicate")}.
\itemize{
\item \code{deprecate_soft()} warns only if the deprecated function is called
directly, i.e. a user is calling a function they wrote in the global
environment or a developer is calling it in their package. It does not
warn when called indirectly, i.e. the deprecation comes from code that
you don't control.
\item \code{deprecate_warn()} warns unconditionally.
\item \code{deprecate_stop()} fails unconditionally.
}

Warnings are only issued once every 8 hours to avoid overwhelming
the user. Control with \code{\link[=verbosity]{options(lifecycle_verbosity)}}.
}
\section{Conditions}{

\itemize{
\item Deprecation warnings have class \code{lifecycle_warning_deprecated}.
\item Deprecation errors have class \code{lifecycle_error_deprecated}.
}
}

\examples{
# A deprecated function `foo`:
deprecate_warn("1.0.0", "foo()")

# A deprecated argument `arg`:
deprecate_warn("1.0.0", "foo(arg)")

# A partially deprecated argument `arg`:
deprecate_warn("1.0.0", "foo(arg = 'must be a scalar integer')")

# A deprecated function with a function replacement:
deprecate_warn("1.0.0", "foo()", "bar()")

# A deprecated function with a function replacement from a
# different package:
deprecate_warn("1.0.0", "foo()", "otherpackage::bar()")

# A deprecated function with custom message:
deprecate_warn(
  when = "1.0.0",
  what = "foo()",
  details = "Please use `otherpackage::bar(foo = TRUE)` instead"
)

# A deprecated function with custom bulleted list:
deprecate_warn(
  when = "1.0.0",
  what = "foo()",
  details = c(
    x = "This is dangerous",
    i = "Did you mean `safe_foo()` instead?"
  )
)
}
\seealso{
\code{\link[=lifecycle]{lifecycle()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg.R
\name{deprecated}
\alias{deprecated}
\alias{is_present}
\title{Mark an argument as deprecated}
\usage{
deprecated()

is_present(arg)
}
\arguments{
\item{arg}{A \code{deprecated()} function argument.}
}
\description{
Signal deprecated argument by using self-documenting sentinel
\code{deprecated()} as default argument. Test whether the caller has
supplied the argument with \code{is_present()}.
}
\section{Magical defaults}{


We recommend importing \code{lifecycle::deprecated()} in your namespace
and use it without the namespace qualifier.

In general, we advise against such magical defaults, i.e. defaults
that cannot be evaluated by the user. In the case of
\code{deprecated()}, the trade-off is worth it because the meaning of
this default is obvious and there is no reason for the user to call
\code{deprecated()} themselves.
}

\examples{
foobar_adder <- function(foo, bar, baz = deprecated()) {
  # Check if user has supplied `baz` instead of `bar`
  if (lifecycle::is_present(baz)) {

    # Signal the deprecation to the user
    deprecate_warn("1.0.0", "foo::bar_adder(baz = )", "foo::bar_adder(bar = )")

    # Deal with the deprecated argument for compatibility
    bar <- baz
  }

  foo + bar
}

foobar_adder(1, 2)
foobar_adder(1, baz = 2)
}
